! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! The Reaction Rates File
! 
! Generated by KPP-2.5.0 symbolic chemistry Kinetics PreProcessor
!       (https:/github.com/KineticPreProcessor/KPP
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian,   Villanova University, USA
!        R. Sander,   Max-Planck Institute for Chemistry, Mainz, Germany
!        M. Long,     Harvard University, USA
!        H. Lin,      Harvard University, USA
!        R. Yantosca, Harvard University, USA
! 
! File                 : gckpp_Rates.F90
! Equation file        : gckpp.kpp
! Output root filename : gckpp
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE gckpp_Rates

  USE gckpp_Parameters
  USE gckpp_Global
  IMPLICIT NONE

CONTAINS



! Begin Rate Law Functions from KPP_HOME/util/UserRateLaws

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  User-defined Rate Law functions
!  Note: the default argument type for rate laws, as read from the equations file, is single precision
!        but all the internal calculations are performed in double precision
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!~~~>  Arrhenius
   REAL(kind=dp) FUNCTION ARR( A0,B0,C0 )
      REAL A0,B0,C0      
      ARR =  DBLE(A0) * EXP(-DBLE(B0)/TEMP) * (TEMP/300.0_dp)**DBLE(C0)
   END FUNCTION ARR        

!~~~> Simplified Arrhenius, with two arguments
!~~~> Note: The argument B0 has a changed sign when compared to ARR
   REAL(kind=dp) FUNCTION ARR2( A0,B0 )
      REAL A0,B0           
      ARR2 =  DBLE(A0) * EXP( DBLE(B0)/TEMP )              
   END FUNCTION ARR2          

   REAL(kind=dp) FUNCTION EP2(A0,C0,A2,C2,A3,C3)
      REAL A0,C0,A2,C2,A3,C3
      REAL(kind=dp) K0,K2,K3            
      K0 = DBLE(A0) * EXP(-DBLE(C0)/TEMP)
      K2 = DBLE(A2) * EXP(-DBLE(C2)/TEMP)
      K3 = DBLE(A3) * EXP(-DBLE(C3)/TEMP)
      K3 = K3*CFACTOR*1.0E6_dp
      EP2 = K0 + K3/(1.0_dp+K3/K2 )
   END FUNCTION EP2

   REAL(kind=dp) FUNCTION EP3(A1,C1,A2,C2) 
      REAL A1, C1, A2, C2
      REAL(kind=dp) K1, K2      
      K1 = DBLE(A1) * EXP(-DBLE(C1)/TEMP)
      K2 = DBLE(A2) * EXP(-DBLE(C2)/TEMP)
      EP3 = K1 + K2*(1.0E6_dp*CFACTOR)
   END FUNCTION EP3 

   REAL(kind=dp) FUNCTION FALL ( A0,B0,C0,A1,B1,C1,CF)
      REAL A0,B0,C0,A1,B1,C1,CF
      REAL(kind=dp) K0, K1     
      K0 = DBLE(A0) * EXP(-DBLE(B0)/TEMP)* (TEMP/300.0_dp)**DBLE(C0)
      K1 = DBLE(A1) * EXP(-DBLE(B1)/TEMP)* (TEMP/300.0_dp)**DBLE(C1)
      K0 = K0*CFACTOR*1.0E6_dp
      K1 = K0/K1
      FALL = (K0/(1.0_dp+K1))*   &
           DBLE(CF)**(1.0_dp/(1.0_dp+(LOG10(K1))**2))
   END FUNCTION FALL

  !---------------------------------------------------------------------------

  ELEMENTAL REAL(kind=dp) FUNCTION k_3rd(temp,cair,k0_300K,n,kinf_300K,m,fc)

    INTRINSIC LOG10

    REAL(kind=dp), INTENT(IN) :: temp      ! temperature [K]
    REAL(kind=dp), INTENT(IN) :: cair      ! air concentration [molecules/cm3]
    REAL, INTENT(IN) :: k0_300K   ! low pressure limit at 300 K
    REAL, INTENT(IN) :: n         ! exponent for low pressure limit
    REAL, INTENT(IN) :: kinf_300K ! high pressure limit at 300 K
    REAL, INTENT(IN) :: m         ! exponent for high pressure limit
    REAL, INTENT(IN) :: fc        ! broadening factor (usually fc=0.6)
    REAL(kind=dp) :: zt_help, k0_T, kinf_T, k_ratio

    zt_help = 300._dp/temp
    k0_T    = k0_300K   * zt_help**(n) * cair ! k_0   at current T
    kinf_T  = kinf_300K * zt_help**(m)        ! k_inf at current T
    k_ratio = k0_T/kinf_T
    k_3rd   = k0_T/(1._dp+k_ratio)*fc**(1._dp/(1._dp+LOG10(k_ratio)**2))

  END FUNCTION k_3rd

  !---------------------------------------------------------------------------

  ELEMENTAL REAL(kind=dp) FUNCTION k_arr (k_298,tdep,temp)
    ! Arrhenius function

    REAL,     INTENT(IN) :: k_298 ! k at T = 298.15K
    REAL,     INTENT(IN) :: tdep  ! temperature dependence
    REAL(kind=dp), INTENT(IN) :: temp  ! temperature

    INTRINSIC EXP

    k_arr = k_298 * EXP(tdep*(1._dp/temp-3.3540E-3_dp)) ! 1/298.15=3.3540e-3

  END FUNCTION k_arr

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  End of User-defined Rate Law functions
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

! End Rate Law Functions from KPP_HOME/util/UserRateLaws


! Begin INLINED Rate Law Functions


  ! All rates are included in Hg_RateLawFuncs.F90, which
  ! gets referenced directly from subroutine Update_Rconst.

! End INLINED Rate Law Functions

! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Update_SUN - update SUN light using TIME
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  SUBROUTINE Update_SUN()
    USE gckpp_Parameters
    USE gckpp_Global

    IMPLICIT NONE

    REAL(kind=dp) :: SunRise, SunSet
    REAL(kind=dp) :: Thour, Tlocal, Ttmp 
    ! PI - Value of pi
    REAL(kind=dp), PARAMETER :: PI = 3.14159265358979d0
    
    SunRise = 4.5_dp 
    SunSet  = 19.5_dp 
    Thour = TIME/3600.0_dp 
    Tlocal = Thour - (INT(Thour)/24)*24

    IF ((Tlocal>=SunRise).AND.(Tlocal<=SunSet)) THEN
       Ttmp = (2.0*Tlocal-SunRise-SunSet)/(SunSet-SunRise)
       IF (Ttmp.GT.0) THEN
          Ttmp =  Ttmp*Ttmp
       ELSE
          Ttmp = -Ttmp*Ttmp
       END IF
       SUN = ( 1.0_dp + COS(PI*Ttmp) )/2.0_dp 
    ELSE
       SUN = 0.0_dp 
    END IF

 END SUBROUTINE Update_SUN

! End of Update_SUN function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Update_RCONST - function to update rate constants
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE Update_RCONST ( )




! Begin INLINED RCONST


  ! Inline an include file containing rate law definitions, which
  ! will be inserted directly into subroutine Update_Rconst().
  ! This is necessary as a workaround for KPP not being able to
  ! include very large files ( > 200000 chars) directly.
  !  -- Bob Yantosca (09 Dec 2021)
  USE Hg_RateLawFuncs

! End INLINED RCONST

  RCONST(1) = (GCARR_ab(1.46d-32,1.86d0)*NUMDEN)
  RCONST(2) = (GCARR_abc(1.6d-9,1.86d0,-7801.0d0)*NUMDEN)
  RCONST(3) = (3.9d-11)
  RCONST(4) = (3.0d-12)
  RCONST(5) = (GCJPLPR_abab(4.3d-30,5.9d0,1.2d-10,1.90d0,0.6d0))
  RCONST(6) = (GCJPLPR_abab(4.3d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(7) = (GCJPLPR_abab(4.3d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(8) = (GCJPLPR_abab(4.3d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(9) = (GCJPLPR_abab(4.3d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(10) = (3.0d-11)
  RCONST(11) = (GCARR_ac(4.1d-12,-856.0d0))
  RCONST(12) = (GCARR_ac(6.0d-11,-550.0d0))
  RCONST(13) = (3.0d-11)
  RCONST(14) = (GCARR_ac(2.25d-33,680.0d0)*NUMDEN)
  RCONST(15) = (3.9d-11)
  RCONST(16) = (3.0d-12)
  RCONST(17) = (GCJPLPR_abab(4.3d-30,5.9d0,1.2d-10,1.90d0,0.6d0))
  RCONST(18) = (GCJPLPR_abab(4.3d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(19) = (GCJPLPR_abab(4.3d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(20) = (GCJPLPR_abab(4.3d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(21) = (3.0d-11)
  RCONST(22) = (3.0d-11)
  RCONST(23) = (3.0d-11)
  RCONST(24) = (GCARR_ac(4.1d-12,-856.0d0))
  RCONST(25) = (GCARR_ac(6.0d-11,-550.0d0))
  RCONST(26) = (GCARR_ac(3.34d-33,43.0d0)*NUMDEN)
  RCONST(27) = (GCARR_ac(1.22d-9,-5720.0d0)*NUMDEN)
  RCONST(28) = (GCJPLPR_abab(4.1d-30,5.9d0,1.2E-10,1.90d0,0.6d0))
  RCONST(29) = (GCJPLPR_abab(4.1d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(30) = (GCJPLPR_abab(4.1d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(31) = (GCJPLPR_abab(4.1d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(32) = (GCJPLPR_abab(4.1d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(33) = (GCJPLPR_abab(4.1d-30,5.9d0,6.9d-11,2.40d0,0.6d0))
  RCONST(34) = (3.0d-11)
  RCONST(35) = (GCARR_ac(4.1d-12,-856.0d0))
  RCONST(36) = (GCARR_ac(6.0d-11,-550.0d0))
  RCONST(37) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgBrNO2)))
  RCONST(38) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgBrHO2)))
  RCONST(39) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgBrOH)))
  RCONST(40) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgBrBrO)))
  RCONST(41) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgBrClO)))
  RCONST(42) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgBr2)))
  RCONST(43) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgClNO2)))
  RCONST(44) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgClHO2)))
  RCONST(45) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgClOH)))
  RCONST(46) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgClBrO)))
  RCONST(47) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgClClO)))
  RCONST(48) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgClBr)))
  RCONST(49) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgCl2)))
  RCONST(50) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgOHNO2)))
  RCONST(51) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgOHHO2)))
  RCONST(52) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgOHOH)))
  RCONST(53) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgOHBrO)))
  RCONST(54) = (Het_HgIIP_Org(State_Het,0.1d0,SR_MW(ind_HgOHClO)))
  RCONST(55) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgBrNO2)))
  RCONST(56) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgBrHO2)))
  RCONST(57) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgBrOH)))
  RCONST(58) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgBrBrO)))
  RCONST(59) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgBrClO)))
  RCONST(60) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgBr2)))
  RCONST(61) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgClNO2)))
  RCONST(62) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgClHO2)))
  RCONST(63) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgClOH)))
  RCONST(64) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgClBrO)))
  RCONST(65) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgClClO)))
  RCONST(66) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgClBr)))
  RCONST(67) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgOHNO2)))
  RCONST(68) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgOHHO2)))
  RCONST(69) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgOHOH)))
  RCONST(70) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgOHBrO)))
  RCONST(71) = (Het_HgIIP_Inorg(State_Het,0.1d0,SR_MW(ind_HgOHClO)))
  RCONST(72) = (PHOTOL(25))
  RCONST(73) = (PHOTOL(26))
  RCONST(74) = (PHOTOL(27))
  RCONST(75) = (PHOTOL(4))
  RCONST(76) = (PHOTOL(7))
  RCONST(77) = (PHOTOL(8))
  RCONST(78) = (PHOTOL(9))
  RCONST(79) = (PHOTOL(10))
  RCONST(80) = (PHOTOL(11))
  RCONST(81) = (PHOTOL(12))
  RCONST(82) = (PHOTOL(14))
  RCONST(83) = (PHOTOL(15))
  RCONST(84) = (PHOTOL(16))
  RCONST(85) = (PHOTOL(17))
  RCONST(86) = (PHOTOL(18))
  RCONST(87) = (PHOTOL(19))
  RCONST(88) = (PHOTOL(6))
  RCONST(89) = (PHOTOL(20))
  RCONST(90) = (PHOTOL(21))
  RCONST(91) = (PHOTOL(5))
  RCONST(92) = (PHOTOL(23))
  RCONST(93) = (PHOTOL(24))
  RCONST(94) = (PHOTOL(13))
      
END SUBROUTINE Update_RCONST

! End of Update_RCONST function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Update_PHOTO - function to update photolytical rate constants
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE Update_PHOTO ( )


   USE gckpp_Global

  RCONST(72) = (PHOTOL(25))
  RCONST(73) = (PHOTOL(26))
  RCONST(74) = (PHOTOL(27))
  RCONST(75) = (PHOTOL(4))
  RCONST(76) = (PHOTOL(7))
  RCONST(77) = (PHOTOL(8))
  RCONST(78) = (PHOTOL(9))
  RCONST(79) = (PHOTOL(10))
  RCONST(80) = (PHOTOL(11))
  RCONST(81) = (PHOTOL(12))
  RCONST(82) = (PHOTOL(14))
  RCONST(83) = (PHOTOL(15))
  RCONST(84) = (PHOTOL(16))
  RCONST(85) = (PHOTOL(17))
  RCONST(86) = (PHOTOL(18))
  RCONST(87) = (PHOTOL(19))
  RCONST(88) = (PHOTOL(6))
  RCONST(89) = (PHOTOL(20))
  RCONST(90) = (PHOTOL(21))
  RCONST(91) = (PHOTOL(5))
  RCONST(92) = (PHOTOL(23))
  RCONST(93) = (PHOTOL(24))
  RCONST(94) = (PHOTOL(13))
      
END SUBROUTINE Update_PHOTO

! End of Update_PHOTO function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



END MODULE gckpp_Rates

