! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! The Reaction Rates File
! 
! Generated by KPP-2.5.0 symbolic chemistry Kinetics PreProcessor
!       (https:/github.com/KineticPreProcessor/KPP
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian,   Villanova University, USA
!        R. Sander,   Max-Planck Institute for Chemistry, Mainz, Germany
!        M. Long,     Harvard University, USA
!        H. Lin,      Harvard University, USA
!        R. Yantosca, Harvard University, USA
! 
! File                 : gckpp_Rates.F90
! Equation file        : gckpp.kpp
! Output root filename : gckpp
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE gckpp_Rates

  USE gckpp_Parameters
  USE gckpp_Global
  IMPLICIT NONE

CONTAINS



! Begin Rate Law Functions from KPP_HOME/util/UserRateLaws

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  User-defined Rate Law functions
!  Note: the default argument type for rate laws, as read from the equations file, is single precision
!        but all the internal calculations are performed in double precision
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!~~~>  Arrhenius
   REAL(kind=dp) FUNCTION ARR( A0,B0,C0 )
      REAL A0,B0,C0      
      ARR =  DBLE(A0) * EXP(-DBLE(B0)/TEMP) * (TEMP/300.0_dp)**DBLE(C0)
   END FUNCTION ARR        

!~~~> Simplified Arrhenius, with two arguments
!~~~> Note: The argument B0 has a changed sign when compared to ARR
   REAL(kind=dp) FUNCTION ARR2( A0,B0 )
      REAL A0,B0           
      ARR2 =  DBLE(A0) * EXP( DBLE(B0)/TEMP )              
   END FUNCTION ARR2          

   REAL(kind=dp) FUNCTION EP2(A0,C0,A2,C2,A3,C3)
      REAL A0,C0,A2,C2,A3,C3
      REAL(kind=dp) K0,K2,K3            
      K0 = DBLE(A0) * EXP(-DBLE(C0)/TEMP)
      K2 = DBLE(A2) * EXP(-DBLE(C2)/TEMP)
      K3 = DBLE(A3) * EXP(-DBLE(C3)/TEMP)
      K3 = K3*CFACTOR*1.0E6_dp
      EP2 = K0 + K3/(1.0_dp+K3/K2 )
   END FUNCTION EP2

   REAL(kind=dp) FUNCTION EP3(A1,C1,A2,C2) 
      REAL A1, C1, A2, C2
      REAL(kind=dp) K1, K2      
      K1 = DBLE(A1) * EXP(-DBLE(C1)/TEMP)
      K2 = DBLE(A2) * EXP(-DBLE(C2)/TEMP)
      EP3 = K1 + K2*(1.0E6_dp*CFACTOR)
   END FUNCTION EP3 

   REAL(kind=dp) FUNCTION FALL ( A0,B0,C0,A1,B1,C1,CF)
      REAL A0,B0,C0,A1,B1,C1,CF
      REAL(kind=dp) K0, K1     
      K0 = DBLE(A0) * EXP(-DBLE(B0)/TEMP)* (TEMP/300.0_dp)**DBLE(C0)
      K1 = DBLE(A1) * EXP(-DBLE(B1)/TEMP)* (TEMP/300.0_dp)**DBLE(C1)
      K0 = K0*CFACTOR*1.0E6_dp
      K1 = K0/K1
      FALL = (K0/(1.0_dp+K1))*   &
           DBLE(CF)**(1.0_dp/(1.0_dp+(LOG10(K1))**2))
   END FUNCTION FALL

  !---------------------------------------------------------------------------

  ELEMENTAL REAL(kind=dp) FUNCTION k_3rd(temp,cair,k0_300K,n,kinf_300K,m,fc)

    INTRINSIC LOG10

    REAL(kind=dp), INTENT(IN) :: temp      ! temperature [K]
    REAL(kind=dp), INTENT(IN) :: cair      ! air concentration [molecules/cm3]
    REAL, INTENT(IN) :: k0_300K   ! low pressure limit at 300 K
    REAL, INTENT(IN) :: n         ! exponent for low pressure limit
    REAL, INTENT(IN) :: kinf_300K ! high pressure limit at 300 K
    REAL, INTENT(IN) :: m         ! exponent for high pressure limit
    REAL, INTENT(IN) :: fc        ! broadening factor (usually fc=0.6)
    REAL(kind=dp) :: zt_help, k0_T, kinf_T, k_ratio

    zt_help = 300._dp/temp
    k0_T    = k0_300K   * zt_help**(n) * cair ! k_0   at current T
    kinf_T  = kinf_300K * zt_help**(m)        ! k_inf at current T
    k_ratio = k0_T/kinf_T
    k_3rd   = k0_T/(1._dp+k_ratio)*fc**(1._dp/(1._dp+LOG10(k_ratio)**2))

  END FUNCTION k_3rd

  !---------------------------------------------------------------------------

  ELEMENTAL REAL(kind=dp) FUNCTION k_arr (k_298,tdep,temp)
    ! Arrhenius function

    REAL,     INTENT(IN) :: k_298 ! k at T = 298.15K
    REAL,     INTENT(IN) :: tdep  ! temperature dependence
    REAL(kind=dp), INTENT(IN) :: temp  ! temperature

    INTRINSIC EXP

    k_arr = k_298 * EXP(tdep*(1._dp/temp-3.3540E-3_dp)) ! 1/298.15=3.3540e-3

  END FUNCTION k_arr

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!  End of User-defined Rate Law functions
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

! End Rate Law Functions from KPP_HOME/util/UserRateLaws


! Begin INLINED Rate Law Functions


  ! All rates are included in fullchem_RateLawFuncs.F90, which
  ! gets referenced directly from subroutine Update_Rconst.

! End INLINED Rate Law Functions

! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Update_SUN - update SUN light using TIME
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  SUBROUTINE Update_SUN()
    USE gckpp_Parameters
    USE gckpp_Global

    IMPLICIT NONE

    REAL(kind=dp) :: SunRise, SunSet
    REAL(kind=dp) :: Thour, Tlocal, Ttmp 
    ! PI - Value of pi
    REAL(kind=dp), PARAMETER :: PI = 3.14159265358979d0
    
    SunRise = 4.5_dp 
    SunSet  = 19.5_dp 
    Thour = TIME/3600.0_dp 
    Tlocal = Thour - (INT(Thour)/24)*24

    IF ((Tlocal>=SunRise).AND.(Tlocal<=SunSet)) THEN
       Ttmp = (2.0*Tlocal-SunRise-SunSet)/(SunSet-SunRise)
       IF (Ttmp.GT.0) THEN
          Ttmp =  Ttmp*Ttmp
       ELSE
          Ttmp = -Ttmp*Ttmp
       END IF
       SUN = ( 1.0_dp + COS(PI*Ttmp) )/2.0_dp 
    ELSE
       SUN = 0.0_dp 
    END IF

 END SUBROUTINE Update_SUN

! End of Update_SUN function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Update_RCONST - function to update rate constants
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE Update_RCONST ( )




! Begin INLINED RCONST


  ! Inline an include file containing rate law definitions, which
  ! will be inserted directly into subroutine Update_Rconst().
  ! This is necessary as a workaround for KPP not being able to
  ! include very large files ( > 200000 chars) directly.
  !  -- Bob Yantosca (11 Jun 2021)
  USE fullchem_RateLawFuncs

! End INLINED RCONST

  RCONST(1) = (K_MT(1))
  RCONST(2) = (K_MT(2))
  RCONST(3) = (K_MT(3))
  RCONST(4) = (K_MT(4))
  RCONST(5) = (K_MT(5))
  RCONST(6) = (K_MT(6))
  RCONST(7) = (K_CLD(1))
  RCONST(8) = (K_CLD(2))
  RCONST(9) = (K_CLD(3))
  RCONST(10) = (K_CLD(4))
  RCONST(11) = (K_CLD(5))
  RCONST(12) = (K_CLD(6))
  RCONST(13) = (GCARR_ac(3.00d-12,-1500.0d0))
  RCONST(14) = (GCARR_ac(1.70d-12,-940.0d0))
  RCONST(15) = (GCARR_ac(1.00d-14,-490.0d0))
  RCONST(16) = (GCARR_ac(1.20d-13,-2450.0d0))
  RCONST(17) = (GCARR_ac(2.90d-16,-1000.0d0))
  RCONST(18) = (1.80d-12)
  RCONST(19) = (GCJPLPR_aba(6.90d-31,1.0d+00,2.6d-11,0.6d0))
  RCONST(20) = (GCARR_ac(4.80d-11,250.0d0))
  RCONST(21) = (1.80d-12)
  RCONST(22) = (GCARR_ac(3.30d-12,270.0d0))
  RCONST(23) = (GC_HO2HO2_acac(3.00d-13,460.0d0,2.1d-33,920.0d0))
  RCONST(24) = (GC_OHCO_a(1.50d-13))
  RCONST(25) = (GCARR_ac(2.45d-12,-1775.0d0))
  RCONST(26) = (GC_RO2NO_B1_ac(2.80d-12,300.0d0))
  RCONST(27) = (GC_RO2NO_A1_ac(2.80d-12,300.0d0))
  RCONST(28) = (GCARR_abc(4.10d-13,0.0d0,750.0d0))
  RCONST(29) = (GC_TBRANCH_1_acac(9.50d-14,390.0d0,2.62d1,-1130.0d0))
  RCONST(30) = (GC_TBRANCH_1_acac(9.50d-14,390.0d0,4.0d-2,1130.0d0))
  RCONST(31) = (1.60d-10)
  RCONST(32) = (GCARR_ac(2.66d-12,200.0d0))
  RCONST(33) = (GCARR_ac(1.14d-12,200.0d0))
  RCONST(34) = (GCARR_ac(2.66d-12,200.0d0))
  RCONST(35) = (GCARR_ac(1.14d-12,200.0d0))
  RCONST(36) = (GCARR_ac(5.50d-12,125.0d0))
  RCONST(37) = (GCJPLPR_aba(1.80d-30,3.0d+00,2.8d-11,0.6d0))
  RCONST(38) = (GC_OHHNO3_acacac(2.41d-14,460.0d0,2.69d-17,2199.0d0,6.51d-34,1335.0d0))
  RCONST(39) = (GCJPLPR_abab(7.00d-31,2.6d+00,3.60d-11,0.1d0,0.6d0))
  RCONST(40) = (GCARR_ac(1.80d-11,-390.0d0))
  RCONST(41) = (GCJPLPR_abab(1.90d-31,3.4d+00,4.0d-12,0.3d0,0.6d0))
  RCONST(42) = (GCJPLPR_abcabc(9.05d-05,3.4d0,-10900.0d0,1.90d15,0.3d0,-10900.0d0,0.6d0))
  RCONST(43) = (GCARR_ac(1.30d-12,380.0d0))
  RCONST(44) = (3.50d-12)
  RCONST(45) = (GCARR_ac(1.50d-11,170.0d0))
  RCONST(46) = (2.20d-11)
  RCONST(47) = (GCJPLPR_abab(2.40d-30,3.0d+00,1.6d-12,-0.1d0,0.6d0))
  RCONST(48) = (GCJPLPR_abcabc(4.14d-04,3.0d0,-10840.0d0,2.76d14,-0.1d0,-10840.0d0,0.6d0))
  RCONST(49) = (4.00d-13)
  RCONST(50) = (GCARR_ac(2.90d-12,-345.0d0))
  RCONST(51) = (GCARR_ac(4.50d-14,-1260.0d0))
  RCONST(52) = (5.80d-16)
  RCONST(53) = (GCARR_ac(4.63d-12,350.0d0))
  RCONST(54) = (GCARR_ac(1.40d-12,-1900.0d0))
  RCONST(55) = (GCJPLPR_abab(9.70d-29,5.6d+00,9.3d-12,1.5d0,0.6d0))
  RCONST(56) = (GCJPLEQ_acabab(9.30d-29,14000.0d0,9.7d-29,5.6d0,9.3d-12,1.5d0,0.6d0))
  RCONST(57) = (GCARR_ac(8.10d-12,270.0d0))
  RCONST(58) = (GCARR_ac(7.66d-12,-1020.0d0))
  RCONST(59) = (GC_RO2NO_B2_aca(2.60d-12,365.0d0,2.0d0))
  RCONST(60) = (GC_RO2NO_A2_aca(2.60d-12,365.0d0,2.0d0))
  RCONST(61) = (GCARR_ac(2.60d-12,365.0d0))
  RCONST(62) = (GC_TBRANCH_2_acabc(7.60d-12,-585.0d0,5.87d0,0.64d0,-816.0d0))
  RCONST(63) = (GC_TBRANCH_2_acabc(7.60d-12,-585.0d0,1.7d-1,-0.64d0,816.0d0))
  RCONST(64) = (GC_RO2NO_B2_aca(2.90d-12,350.0d0,3.0d0))
  RCONST(65) = (GC_RO2NO_A2_aca(2.90d-12,350.0d0,3.0d0))
  RCONST(66) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(67) = (GCARR_ac(9.10d-12,-405.0d0))
  RCONST(68) = (GC_RO2NO_B2_aca(2.70d-12,350.0d0,4.5d0))
  RCONST(69) = (GC_RO2NO_A2_aca(2.70d-12,350.0d0,4.5d0))
  RCONST(70) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(71) = (GCARR_ac(2.80d-12,300.0d0))
  RCONST(72) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(73) = (GC_RO2NO_B2_aca(2.70d-12,360.0d0,3.0d0))
  RCONST(74) = (GC_RO2NO_A2_aca(2.70d-12,360.0d0,3.0d0))
  RCONST(75) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(76) = (GCARR_ac(2.80d-12,-3280.0d0))
  RCONST(77) = (1.60d-12)
  RCONST(78) = (GCARR_ac(3.15d-14,920.0d0))
  RCONST(79) = (GCARR_ac(6.00d-12,410.0d0))
  RCONST(80) = (GCJPLPR_abab(9.00d-28,8.9d0,7.7d-12,0.2d0,0.6d0))
  RCONST(81) = (GCJPLEQ_acabab(9.00d-29,14000.0d0,9.00d-28,8.9d0,7.7d-12,0.2d0,0.6d0))
  RCONST(82) = (GCARR_ac(6.70d-12,340.0d0))
  RCONST(83) = (6.50d-15)
  RCONST(84) = (1.33d-13+3.82d-11*exp(-2000.0d0/TEMP))
  RCONST(85) = (5.92d-13)
  RCONST(86) = (5.92d-13)
  RCONST(87) = (GCARR_ac(7.40d-13,700.0d0))
  RCONST(88) = (GCARR_ac(7.40d-13,700.0d0))
  RCONST(89) = (GCARR_ac(8.60d-13,700.0d0))
  RCONST(90) = (GC_RO2HO2_aca(2.91d-13,1300.0d0,4.0d0))
  RCONST(91) = (GC_RO2HO2_aca(2.91d-13,1300.0d0,3.0d0))
  RCONST(92) = (GC_RO2HO2_aca(2.91d-13,1300.0d0,3.0d0))
  RCONST(93) = (GCARR_ac(1.30d-12,-25.0d0))
  RCONST(94) = (3.00d-13)
  RCONST(95) = (3.00d-13)
  RCONST(96) = (8.00d-16)
  RCONST(97) = (8.37d-14)
  RCONST(98) = (8.37d-14)
  RCONST(99) = (GCARR_ac(7.50d-13,500.0d0))
  RCONST(100) = (8.37d-14)
  RCONST(101) = (8.37d-14)
  RCONST(102) = (8.37d-14)
  RCONST(103) = (3.35d-12)
  RCONST(104) = (GCARR_ac(4.60d-12,70.0d0))
  RCONST(105) = (4.10d-14)
  RCONST(106) = (4.10d-14)
  RCONST(107) = (2.70d-14)
  RCONST(108) = (2.70d-14)
  RCONST(109) = (GCARR_ac(7.40d-13,700.0d0))
  RCONST(110) = (GCARR_ac(7.40d-13,700.0d0))
  RCONST(111) = (GC_RO2HO2_aca(2.91d-13,1300.0d0,3.0d0))
  RCONST(112) = (GC_RO2HO2_aca(2.91d-13,1300.0d0,3.0d0))
  RCONST(113) = (GCARR_ac(4.30d-13,1040.0d0))
  RCONST(114) = (GCJPLPR_abab(4.60d-27,4.0d0,2.6d-11,1.3d0,0.5d0))
  RCONST(115) = (GCARR_ac(5.50d-15,-1880.0d0))
  RCONST(116) = (GC_GLYCOH_A_a(8.00d-12))
  RCONST(117) = (GC_GLYCOH_B_a(8.00d-12))
  RCONST(118) = (GCARR_ac(4.59d-13,-1156.0d0))
  RCONST(119) = (GCARR_ac(3.10d-12,340.0d0))
  RCONST(120) = (1.50d-11)
  RCONST(121) = (GC_GLYXNO3_ac(1.40d-12,-1860.0d0))
  RCONST(122) = (GCARR_ac(3.36d-12,-1860.0d0))
  RCONST(123) = (GC_HACOH_A_ac(2.15d-12,305.0d0))
  RCONST(124) = (GC_HACOH_B_ac(2.15d-12,305.0d0))
  RCONST(125) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(126) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(127) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(128) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(129) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(130) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(131) = (GCARR_ac(8.78d-12,200.0d0))
  RCONST(132) = (GCARR_ac(5.18d-12,200.0d0))
  RCONST(133) = (GCARR_ac(5.18d-12,200.0d0))
  RCONST(134) = (GCARR_ac(8.78d-12,200.0d0))
  RCONST(135) = (GCARR_ac(8.78d-12,200.0d0))
  RCONST(136) = (GCARR_ac(6.13d-13,200.0d0))
  RCONST(137) = (GCARR_ac(8.78d-12,200.0d0))
  RCONST(138) = (GCARR_ac(4.82d-11,-400.0d0))
  RCONST(139) = (GCARR_ac(6.13d-13,200.0d0))
  RCONST(140) = (1.40d-18)
  RCONST(141) = (GCARR_ac(2.50d-12,500.0d0))
  RCONST(142) = (GCARR_ac(1.80d-12,500.0d0))
  RCONST(143) = (GCARR_ac(2.00d-13,500.0d0))
  RCONST(144) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(145) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(146) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(147) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(148) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(149) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(150) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(151) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(152) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(153) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(154) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(155) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(156) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(157) = (GCARR_ac(1.68d-12,500.0d0))
  RCONST(158) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(159) = (GCARR_ac(1.87d-13,500.0d0))
  RCONST(160) = (GCARR_ac(2.50d-12,500.0d0))
  RCONST(161) = (GCARR_ac(8.50d-13,-2450.0d0))
  RCONST(162) = (GCJPLPR_abab(1.00d-30,4.8d+00,7.2d-12,2.1d0,0.6d0))
  RCONST(163) = (GCJPLPR_abcabc(1.05d-02,4.8d+00,-11234.0d0,7.58d16,2.1d0,-11234.0d0,0.6d0))
  RCONST(164) = (GCARR_ac(1.20d-11,-280.0d0))
  RCONST(165) = (GC_DMSOH_acac(8.20d-39,5376.0d0,1.05d-5,3644.0d0))
  RCONST(166) = (GCARR_ac(1.90d-13,530.0d0))
  RCONST(167) = (GCJPLPR_aba(3.30d-31,4.3d+00,1.6d-12,0.6d0))
  RCONST(168) = (GCARR_ac(1.60d-11,-780.0d0))
  RCONST(169) = (GCARR_ac(4.50d-12,460.0d0))
  RCONST(170) = (GCARR_ac(4.80d-12,-310.0d0))
  RCONST(171) = (GCARR_ac(5.50d-12,200.0d0))
  RCONST(172) = (GCARR_ac(2.40d-12,40.0d0))
  RCONST(173) = (GCARR_ac(2.80d-14,860.0d0))
  RCONST(174) = (GCARR_ac(8.80d-12,260.0d0))
  RCONST(175) = (4.90d-11)
  RCONST(176) = (GCARR_ac(2.10d-11,240.0d0))
  RCONST(177) = (GCARR_ac(1.20d-10,-430.0d0))
  RCONST(178) = (GCARR_ac(5.80d-12,-1500.0d0))
  RCONST(179) = (GCARR_ac(1.70d-11,250.0d0))
  RCONST(180) = (1.60d-11)
  RCONST(181) = (GCARR_ac(1.70d-11,-800.0d0))
  RCONST(182) = (GCARR_ac(1.80d-11,-460.0d0))
  RCONST(183) = (GCARR_ac(1.66d-10,-7000.0d0))
  RCONST(184) = (GCARR_ac(2.36d-10,-6411.0d0))
  RCONST(185) = (GCARR_ac(8.77d-11,-4330.0d0))
  RCONST(186) = (GCJPLPR_aba(4.20d-31,2.4d0,2.7d-11,0.6d0))
  RCONST(187) = (GCJPLPR_abab(5.40d-31,3.1d0,6.5d-12,2.9d0,0.6d0))
  RCONST(188) = (GCARR_ac(9.00d-13,-360.0d0))
  RCONST(189) = (GCARR_ac(2.00d-12,-840.0d0))
  RCONST(190) = (GCARR_ac(1.42d-12,-1150.0d0))
  RCONST(191) = (GCARR_ac(1.63d-10,60.0d0))
  RCONST(192) = (GCARR_ac(2.15d-11,110.0d0))
  RCONST(193) = (GCARR_ac(3.30d-11,55.0d0))
  RCONST(194) = (1.20d-10)
  RCONST(195) = (GCARR_ac(4.63d-11,20.0d0))
  RCONST(196) = (GCARR_ac(7.25d-11,20.0d0))
  RCONST(197) = (1.31d-10)
  RCONST(198) = (0.09d-10)
  RCONST(199) = (0.35d-10)
  RCONST(200) = (GCARR_ab(6.00d-34,2.4d0)*NUMDEN)
  RCONST(201) = (GCARR_ac(8.00d-12,-2060.0d0))
  RCONST(202) = (GCARR_ac(2.80d-12,-1800.0d0))
  RCONST(203) = (GCARR_ac(1.80d-11,180.0d0))
  RCONST(204) = (GCARR_ac(3.00d-11,200.0d0))
  RCONST(205) = (1.20d-10)
  RCONST(206) = (1.20d-10)
  RCONST(207) = (GCARR_ac(2.10d-11,-2200.0d0))
  RCONST(208) = (GCARR_ac(1.10d-13,-1200.0d0))
  RCONST(209) = (GCARR_ac(5.10d-12,210.0d0))
  RCONST(210) = (1.00d-11)
  RCONST(211) = (GCJPLPR_aba(9.00d-32,1.5d+00,3.0d-11,0.6d0))
  RCONST(212) = (GCJPLPR_abab(2.50d-31,1.8d+00,2.2d-11,0.7d0,0.6d0))
  RCONST(213) = (GCARR_ac(1.40d-12,-2000.0d0))
  RCONST(214) = (GCJPLPR_abab(4.40d-32,1.3d+00,7.5d-11,-0.2d0,0.6d0))
  RCONST(215) = (GCARR_ac(1.40d-10,-470.0d0))
  RCONST(216) = (7.20d-11)
  RCONST(217) = (1.60d-12)
  RCONST(218) = (6.90d-12)
  RCONST(219) = (GCARR_ac(1.50d-11,-3600.0d0))
  RCONST(220) = (GCARR_ac(2.10d-11,100.0d0))
  RCONST(221) = (GCARR_ac(5.80d-12,220.0d0))
  RCONST(222) = (GCARR_ac(1.90d-11,230.0d0))
  RCONST(223) = (GCARR_ac(3.40d-11,-1600.0d0))
  RCONST(224) = (1.50d-10)
  RCONST(225) = (1.50d-10)
  RCONST(226) = (2.70d-10)
  RCONST(227) = (3.30d-10)
  RCONST(228) = (1.80d-10)
  RCONST(229) = (2.70d-10)
  RCONST(230) = (6.60d-10)
  RCONST(231) = (1.02d-10)
  RCONST(232) = (2.30d-10)
  RCONST(233) = (1.40d-10)
  RCONST(234) = (1.50d-10)
  RCONST(235) = (1.00d-10)
  RCONST(236) = (2.60d-10)
  RCONST(237) = (2.00d-10)
  RCONST(238) = (2.00d-10)
  RCONST(239) = (2.32d-10)
  RCONST(240) = (GCARR_ac(1.30d-10,-25.0d0))
  RCONST(241) = (GCARR_ac(5.40d-11,-30.0d0))
  RCONST(242) = (GCARR_ac(1.60d-10,0.0d0))
  RCONST(243) = (GCARR_ac(2.60d-12,-1100.0d0))
  RCONST(244) = (GCARR_ac(1.80d-11,-600.0d0))
  RCONST(245) = (GCARR_ac(7.40d-12,270.0d0))
  RCONST(246) = (GCARR_ac(6.00d-13,230.0d0))
  RCONST(247) = (GCARR_ac(1.40d-12,600.0d0))
  RCONST(248) = (GCARR_ac(6.00d-13,670.0d0))
  RCONST(249) = (GCARR_ac(1.80d-12,-250.0d0))
  RCONST(250) = (GCARR_ac(3.00d-12,-500.0d0))
  RCONST(251) = (GCARR_ac(2.40d-12,-1250.0d0))
  RCONST(252) = (GCARR_ac(1.20d-12,-330.0d0))
  RCONST(253) = (GCARR_ac(1.96d-12,-1200.0d0))
  RCONST(254) = (GCARR_ac(2.61d-12,-944.0d0))
  RCONST(255) = (GCARR_ac(4.69d-12,-1134.0d0))
  RCONST(256) = (GCARR_ac(1.64d-12,-1520.0d0))
  RCONST(257) = (GCARR_ac(9.20d-13,-1560.0d0))
  RCONST(258) = (GCARR_ac(1.25d-12,-1600.0d0))
  RCONST(259) = (GCARR_ac(1.30d-12,-1770.0d0))
  RCONST(260) = (GCARR_ac(7.40d-13,-900.0d0))
  RCONST(261) = (GCARR_ac(7.10d-12,-1270.0d0))
  RCONST(262) = (GCARR_ac(7.32d-11,-30.0d0))
  RCONST(263) = (GCARR_ac(2.30d-11,-200.0d0))
  RCONST(264) = (GCARR_ac(3.05d-11,-2270.0d0))
  RCONST(265) = (GCARR_ac(1.10d-11,-980.0d0))
  RCONST(266) = (GCARR_ac(1.40d-11,270.0d0))
  RCONST(267) = (GCARR_ac(3.60d-11,-375.0d0))
  RCONST(268) = (GCARR_ac(2.80d-11,85.0d0))
  RCONST(269) = (GCARR_ac(2.60d-12,290.0d0))
  RCONST(270) = (GCARR_ac(6.40d-12,290.0d0))
  RCONST(271) = (GCJPLPR_abab(1.80d-31,3.4d+00,1.50d-11,1.9d0,0.6d0))
  RCONST(272) = (GCARR_ac(1.00d-12,-1590.0d0))
  RCONST(273) = (GCARR_ac(3.00d-11,-2450.0d0))
  RCONST(274) = (GCARR_ac(3.50d-13,-1370.0d0))
  RCONST(275) = (GCJPLPR_aba(2.20d-33,3.1d+00,1.8d-10,0.6d0))
  RCONST(276) = (GCJPLEQ_acabab(6.60d-25,2502.0d0,2.20d-33,3.1d+00,1.8d-10,0.0d0,0.6d0))
  RCONST(277) = (GCJPLPR_abab(1.90d-32,3.6d+00,3.7d-12,1.6d0,0.6d0))
  RCONST(278) = (GCJPLEQ_acabab(2.16d-27,8537.0d0,1.90d-32,3.6d+00,3.7d-12,1.6d0,0.6d0))
  RCONST(279) = (2.30d-10)
  RCONST(280) = (1.20d-11)
  RCONST(281) = (GCARR_ac(9.50d-13,550.0d0))
  RCONST(282) = (GCARR_ac(2.30d-12,260.0d0))
  RCONST(283) = (GCARR_ac(4.10d-13,290.0d0))
  RCONST(284) = (GCARR_ac(3.60d-12,-840.0d0))
  RCONST(285) = (GCARR_ac(6.50d-12,135.0d0))
  RCONST(286) = (GCARR_ac(2.17d-11,-1130.0d0))
  RCONST(287) = (GCARR_ac(1.24d-12,-1070.0d0))
  RCONST(288) = (GCARR_ac(3.77d-12,-1011.0d0))
  RCONST(289) = (2.00d-13)
  RCONST(290) = (1.60d-10)
  RCONST(291) = (5.7d-11)
  RCONST(292) = (GCARR_ac(7.2d-11,-70.0d0))
  RCONST(293) = (7.4d-11)
  RCONST(294) = (7.4d-11)
  RCONST(295) = (5.5d-11)
  RCONST(296) = (9.6d-11)
  RCONST(297) = (2.8d-14)
  RCONST(298) = (GCARR_ac(6.54d-11,60.0d0))
  RCONST(299) = (GCARR_ac(8.12d-11,-90.0d0))
  RCONST(300) = (GCARR_ac(7.70d-11,-1000.0d0))
  RCONST(301) = (GCARR_ac(7.60d-11,500.0d0))
  RCONST(302) = (2.05d-10)
  RCONST(303) = (GCJPLPR_aa(4.00d-28,2.8d-10,0.6d0))
  RCONST(304) = (3.60d-12)
  RCONST(305) = (GCJPLPR_aba(1.80d-32,1.0d0,1.77d-11,0.6d0))
  RCONST(306) = (GCARR_ac(8.40d-11,-2620.0d0))
  RCONST(307) = (GCJPLPR_aba(3.00d-31,1.0d0,6.6d-11,0.63d0))
  RCONST(308) = (GCARR_ac(9.94d+17,-11859.0d0))
  RCONST(309) = (GCARR_ac(2.90d-11,-2600.0d0))
  RCONST(310) = (1.50d-12)
  RCONST(311) = (GCJPLPR_abab(7.50d-31,3.5d0,7.6d-12,1.5d0,0.6d0))
  RCONST(312) = (GCARR_ac(2.10d+15,-13670.0d0))
  RCONST(313) = (GCARR_ac(9.10d-11,-146.0d0))
  RCONST(314) = (1.20d-11)
  RCONST(315) = (GCARR_ac(3.00d-12,510.0d0))
  RCONST(316) = (GCARR_ac(1.20d-11,510.0d0))
  RCONST(317) = (1.00d-10)
  RCONST(318) = (1.50d-10)
  RCONST(319) = (3.80d-02)
  RCONST(320) = (GCARR_ac(1.10d-12,542.0d0))
  RCONST(321) = (GCARR_ac(5.10d-12,280.0d0))
  RCONST(322) = (GCARR_ac(2.81d-12,280.0d0))
  RCONST(323) = (GCARR_ac(1.02d-12,280.0d0))
  RCONST(324) = (GCARR_ac(2.30d-11,-870.0d0))
  RCONST(325) = (GCARR_ac(1.50d-11,-1090.0d0))
  RCONST(326) = (1.80d-10)
  RCONST(327) = (3.00d-11)
  RCONST(328) = (5.00d-12)
  RCONST(329) = (GCARR_ac(1.30d-11,570.0d0))
  RCONST(330) = (GCARR_ac(9.10d-12,240.0d0))
  RCONST(331) = (GCARR_ac(6.00d-12,500.0d0))
  RCONST(332) = (GCARR_ac(9.00d-12,500.0d0))
  RCONST(333) = (GCARR_ac(1.00d+12,-9770.0d0))
  RCONST(334) = (GCARR_ac(2.50d+14,-9770.0d0))
  RCONST(335) = (GCARR_ac(2.90d-12,-1100.0d0))
  RCONST(336) = (2.40d-12)
  RCONST(337) = (6.70d-13)
  RCONST(338) = (1.20d-15)
  RCONST(339) = (1.00d-14)
  RCONST(340) = (1.00d-15)
  RCONST(341) = (1.70d-15)
  RCONST(342) = (GCARR_ac(2.88d-35,1391.0d0))
  RCONST(343) = (1.40d-12)
  RCONST(344) = (3.70d-11)
  RCONST(345) = (1.20d-15)
  RCONST(346) = (1.00d-14)
  RCONST(347) = (1.00d-15)
  RCONST(348) = (7.00d-14)
  RCONST(349) = (6.00d-18)
  RCONST(350) = (1.00d-17)
  RCONST(351) = (GCARR_ac(1.21d-11,440.0d0))
  RCONST(352) = (GCARR_ac(1.21d-11,440.0d0))
  RCONST(353) = (4.00d-12)
  RCONST(354) = (1.50d-11)
  RCONST(355) = (GCARR_ac(3.56d-14,708.0d0))
  RCONST(356) = (GCARR_ac(7.40d-13,765.0d0))
  RCONST(357) = (1.20d-12)
  RCONST(358) = (GCARR_ac(5.00d-16,-530.0d0))
  RCONST(359) = (GCARR_ac(5.00d-16,-530.0d0))
  RCONST(360) = (GCARR_ac(8.33d-13,490.0d0))
  RCONST(361) = (GCARR_ac(8.33d-13,490.0d0))
  RCONST(362) = (GCARR_ac(4.20d-11,401.0d0))
  RCONST(363) = (GCARR_ac(2.95d-15,-783.0d0))
  RCONST(364) = (1.22d-11)
  RCONST(365) = (4.00d-12)
  RCONST(366) = (1.50d-11)
  RCONST(367) = (GCARR_ac(3.56d-14,708.0d0))
  RCONST(368) = (GCARR_ac(7.40d-13,765.0d0))
  RCONST(369) = (1.20d-12)
  RCONST(370) = (GCARR_ac(3.40d-12,190.0d0))
  RCONST(371) = (4.00d-12)
  RCONST(372) = (4.00d-12)
  RCONST(373) = (GCARR_ac(1.66d-13,1300.0d0))
  RCONST(374) = (GCARR_ac(1.66d-13,1300.0d0))
  RCONST(375) = (GCARR_ac(1.60d-13,708.0d0))
  RCONST(376) = (GCARR_ac(9.68d-14,708.0d0))
  RCONST(377) = (GCARR_ac(8.85d-13,765.0d0))
  RCONST(378) = (GCARR_ac(5.37d-13,765.0d0))
  RCONST(379) = (1.20d-12)
  RCONST(380) = (1.20d-12)
  RCONST(381) = (GCARR_ac(7.00d-14,1000.0d0))
  RCONST(382) = (GCARR_ac(4.25d-14,1000.0d0))
  RCONST(383) = (GCARR_ac(2.96d-14,1000.0d0))
  RCONST(384) = (4.80d-12)
  RCONST(385) = (7.29d-11)
  RCONST(386) = (1.67d-16)
  RCONST(387) = (GCARR_ac(3.15d-13,-448.0d0))
  RCONST(388) = (GCARR_ac(3.15d-13,-448.0d0))
  RCONST(389) = (2.78d-04)
  RCONST(390) = (2.78d-04)
  RCONST(391) = (GC_OHHNO3_acacac(2.41d-14,460.0d0,2.69d-17,2199.0d0,6.51d-34,1335.0d0))
  RCONST(392) = (GCARR_ac(8.00d-13,-1000.0d0))
  RCONST(393) = (GCARR_ac(1.00d-12,-490.0d0))
  RCONST(394) = (GCARR_ac(1.20d-12,-320.0d0))
  RCONST(395) = (7.10d-13)
  RCONST(396) = (1.3d-17)
  RCONST(397) = (GC_ISO1(1.7d-11,3.90d2,9.33d-2,5.05d15,-1.22d4,1.79d14,-8.830d3))
  RCONST(398) = (GC_ISO1(1.0d-11,3.90d2,2.26d-1,2.22d9,-7.160d3,1.75d14,-9.054d3))
  RCONST(399) = (GC_ISO2(1.7d-11,3.90d2,9.33d-2,5.05d15,-1.22d4,1.79d14,-8.830d3))
  RCONST(400) = (GC_ISO2(1.0d-11,3.90d2,2.26d-1,2.22d9,-7.160d3,1.75d14,-9.054d3))
  RCONST(401) = (ARRPLUS_abde(2.12d-13,-1300d0,1.1644d0,-7.0485d-4))
  RCONST(402) = (ARRPLUS_abde(2.12d-13,-1300d0,-0.1644d0,7.0485d-4))
  RCONST(403) = (ARRPLUS_abde(2.12d-13,-1300d0,1.2038d0,-9.0435d-4))
  RCONST(404) = (ARRPLUS_abde(2.12d-13,-1300d0,-0.2038d0,9.0435d-4))
  RCONST(405) = (ARRPLUS_abde(1.04d11,9.746d3,1.1644d0,-7.0485d-4))
  RCONST(406) = (TUNPLUS_abcde(5.05d15,-1.22d4,1.0d8,-0.0128d0,5.1242d-5))
  RCONST(407) = (ARRPLUS_abde(1.88d11,9.752d3,1.2038d0,-9.0435d-4))
  RCONST(408) = (TUNPLUS_abcde(2.22d9,-7.160d3,1.0d8,-0.0306d0,1.1346d-4))
  RCONST(409) = (ARRPLUS_ade(6.92d-14,1.1644d0,-7.0485d-4))
  RCONST(410) = (ARRPLUS_ade(5.74d-12,1.2038d0,-9.0435d-4))
  RCONST(411) = (ARRPLUS_ade(1.54d-12,2.3682d0,-1.6092d-3))
  RCONST(412) = (ARRPLUS_ade(2.49d-12,-0.1644d0,7.0485d-4))
  RCONST(413) = (ARRPLUS_ade(3.94d-12,-0.2038d0,9.0435d-4))
  RCONST(414) = (ARRPLUS_ade(1.54d-12,-0.3682d0,1.6092d-3))
  RCONST(415) = (ARRPLUS_ade(2.0d-12,1.1644d0,-7.0485d-4))
  RCONST(416) = (ARRPLUS_ade(2.0d-12,-0.1644d0,7.0485d-4))
  RCONST(417) = (ARRPLUS_ade(2.0d-12,1.2038d0,-9.0435d-4))
  RCONST(418) = (ARRPLUS_ade(2.0d-12,-0.2038d0,9.0435d-4))
  RCONST(419) = (GC_NIT(2.7d-12,3.50d2,1.19d0,6.0d0,1.1644d0,7.05d-4))
  RCONST(420) = (GC_ALK(2.7d-12,3.50d2,1.19d0,6.0d0,1.1644d0,7.05d-4))
  RCONST(421) = (GC_NIT(2.7d-12,3.50d2,1.421d0,6.0d0,-0.1644d0,-7.05d-4))
  RCONST(422) = (GC_ALK(2.7d-12,3.50d2,1.421d0,6.0d0,-0.1644d0,-7.05d-4))
  RCONST(423) = (GC_NIT(2.7d-12,3.50d2,1.297d0,6.0d0,1.2038d0,9.04d-4))
  RCONST(424) = (GC_ALK(2.7d-12,3.50d2,1.297d0,6.0d0,1.2038d0,9.04d-4))
  RCONST(425) = (GC_NIT(2.7d-12,3.50d2,1.421d0,6.0d0,-0.2038d0,-9.04d-4))
  RCONST(426) = (GC_ALK(2.7d-12,3.50d2,1.421d0,6.0d0,-0.2038d0,-9.04d-4))
  RCONST(427) = (GCARR_ac(1.17d-11,450.0d0))
  RCONST(428) = (GCARR_ac(1.17d-11,450.0d0))
  RCONST(429) = (GCARR_ac(2.20d-11,390.0d0))
  RCONST(430) = (GCARR_ac(3.50d-11,390.0d0))
  RCONST(431) = (GCARR_ac(4.64d-12,650.0d0))
  RCONST(432) = (GCARR_ac(9.85d-12,410.0d0))
  RCONST(433) = (GCARR_ac(3.00d-12,650.0d0))
  RCONST(434) = (GCARR_ac(2.47d-12,390.0d0))
  RCONST(435) = (GC_EPO_a(1.62d-11,3.90d2,4.77d-21))
  RCONST(436) = (GCARR_ac(4.35d-12,390.0d0))
  RCONST(437) = (GC_EPO_a(2.85d-11,390.0d0,4.77d-21))
  RCONST(438) = (GCARR_ac(6.10d-12,200.0d0))
  RCONST(439) = (GCARR_ac(4.10d-12,200.0d0))
  RCONST(440) = (GCARR_ac(3.53d-11,390.0d0))
  RCONST(441) = (GCARR_ac(3.53d-11,390.0d0))
  RCONST(442) = (GCARR_ac(1.59d+13,-10000.0d0))
  RCONST(443) = (GC_ALK(2.7d-12,3.50d2,2.1d0,9.0d0,1.0d0,0.0d0))
  RCONST(444) = (GC_NIT(2.7d-12,3.50d2,2.1d0,9.0d0,1.0d0,0.0d0))
  RCONST(445) = (GCARR_ac(2.47d-13,1300.0d0))
  RCONST(446) = (GCARR_ac(2.91d+13,-10000.0d0))
  RCONST(447) = (GC_ALK(2.7d-12,3.50d2,2.315d0,9.0d0,1.0d0,0.0d0))
  RCONST(448) = (GC_NIT(2.7d-12,3.50d2,2.315d0,9.0d0,1.0d0,0.0d0))
  RCONST(449) = (GCARR_ac(2.47d-13,1300.0d0))
  RCONST(450) = (GCARR_ac(1.875d+13,-10000.0d0))
  RCONST(451) = (GC_ALK(2.7d-12,3.50d2,3.079d0,9.0d0,1.0d0,0.0d0))
  RCONST(452) = (GC_NIT(2.7d-12,3.50d2,3.079d0,9.0d0,1.0d0,0.0d0))
  RCONST(453) = (GCARR_ac(2.47d-13,1300.0d0))
  RCONST(454) = (GCARR_ac(3.22d-11,-400.0d0))
  RCONST(455) = (GCARR_ac(1.05d-11,-400.0d0))
  RCONST(456) = (GC_EPO_a(5.82d-11,-4.00d2,1.14d-20))
  RCONST(457) = (GCARR_ac(8.25d-12,-400.0d0))
  RCONST(458) = (GC_EPO_a(3.75d-11,-4.00d2,8.91d-21))
  RCONST(459) = (GCARR_ac(1.875d+13,-10000.0d0))
  RCONST(460) = (GCARR_ac(1.0d+7,-5000.0d0))
  RCONST(461) = (GCARR_ac(2.38d-13,1300.0d0))
  RCONST(462) = (GC_ALK(2.7d-12,3.50d2,13.098d0,8.0d0,1.0d0,0.0d0))
  RCONST(463) = (GC_NIT(2.7d-12,3.50d2,13.098d0,8.0d0,1.0d0,0.0d0))
  RCONST(464) = (GCARR_ac(1.875d+13,-10000.0d0))
  RCONST(465) = (GCARR_ac(1.0d+7,-5000.0d0))
  RCONST(466) = (GC_ALK(2.7d-12,3.50d2,16.463d0,8.0d0,1.0d0,0.0d0))
  RCONST(467) = (GC_NIT(2.7d-12,3.50d2,16.463d0,8.0d0,1.0d0,0.0d0))
  RCONST(468) = (GCARR_ac(2.38d-13,1300.0d0))
  RCONST(469) = (GCARR_ac(2.38d-13,1300.0d0))
  RCONST(470) = (GC_NIT(2.7d-12,3.50d2,13.098d0,8.0d0,1.0d0,0.0d0))
  RCONST(471) = (GC_ALK(2.7d-12,3.50d2,13.098d0,8.0d0,1.0d0,0.0d0))
  RCONST(472) = (GCARR_ac(1.875d+13,-10000.0d0))
  RCONST(473) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(474) = (GCARR_ac(2.38d-13,1300.0d0))
  RCONST(475) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(476) = (GCARR_ac(2.38d-13,1300.0d0))
  RCONST(477) = (GCARR_ac(7.14d-12,390.0d0))
  RCONST(478) = (GC_EPO_a(6.30d-12,390.0d0,1.62d-19))
  RCONST(479) = (GCARR_ac(1.02d-11,390.0d0))
  RCONST(480) = (GC_EPO_a(1.05d-11,390.0d0,2.49d-19))
  RCONST(481) = (GC_EPO_a(1.55d-11,390.0d0,2.715d-19))
  RCONST(482) = (GCARR_ac(2.04d-11,390.0d0))
  RCONST(483) = (GC_EPO_a(9.52d-12,390.0d0,2.715d-19))
  RCONST(484) = (GCARR_ac(2.95d-11,390.0d0))
  RCONST(485) = (GCARR_ac(7.5d-12,20.0d0))
  RCONST(486) = (GCARR_ac(7.5d-12,20.0d0))
  RCONST(487) = (GCARR_ac(1.875d+13,-10000.0d0))
  RCONST(488) = (GCARR_ac(2.60d-13,1300.0d0))
  RCONST(489) = (GC_ALK(2.7d-12,350.0d0,6.32d0,11.0d0,1.0d0,0.0d0))
  RCONST(490) = (GC_NIT(2.7d-12,350.0d0,6.32d0,11.0d0,1.0d0,0.0d0))
  RCONST(491) = (GCARR_ac(1.875d+13,-10000.0d0))
  RCONST(492) = (GCARR_ac(2.60d-13,1300.0d0))
  RCONST(493) = (GC_ALK(2.7d-12,350.0d0,7.941d0,11.0d0,1.0d0,0.0d0))
  RCONST(494) = (GC_NIT(2.7d-12,350.0d0,7.941d0,11.0d0,1.0d0,0.0d0))
  RCONST(495) = (GCARR_ac(1.256d+13,-10000.0d0))
  RCONST(496) = (GCARR_ac(5.092d+12,-10000.0d0))
  RCONST(497) = (GCARR_ac(2.60d-13,1300.0d0))
  RCONST(498) = (GC_ALK(2.7d-12,350.0d0,4.712d0,11.0d0,1.0d0,0.0d0))
  RCONST(499) = (GC_NIT(2.7d-12,350.0d0,4.712d0,11.0d0,1.0d0,0.0d0))
  RCONST(500) = (GCARR_ac(2.60d-13,1300.0d0))
  RCONST(501) = (GC_ALK(2.7d-12,350.0d0,2.258d0,11.0d0,1.0d0,0.0d0))
  RCONST(502) = (GC_NIT(2.7d-12,350.0d0,2.258d0,11.0d0,1.0d0,0.0d0))
  RCONST(503) = (GCARR_ac(2.60d-13,1300.0d0))
  RCONST(504) = (GC_ALK(2.7d-12,350.0d0,1.851d0,11.0d0,1.0d0,0.0d0))
  RCONST(505) = (GC_NIT(2.7d-12,350.0d0,1.851d0,11.0d0,1.0d0,0.0d0))
  RCONST(506) = (GCARR_ac(2.95d-12,450.0d0))
  RCONST(507) = (GCARR_ac(2.47d-13,1300.0d0))
  RCONST(508) = (GCARR_ac(2.47d-13,1300.0d0))
  RCONST(509) = (1.61d-12)
  RCONST(510) = (2.56d-12)
  RCONST(511) = (3.71d-12)
  RCONST(512) = (1.18d-12)
  RCONST(513) = (2.80d-13)
  RCONST(514) = (1.92d-12)
  RCONST(515) = (7.71d-12)
  RCONST(516) = (2.3d-12)
  RCONST(517) = (2.3d-12)
  RCONST(518) = (GC_ALK(2.7d-12,350.0d0,12.915d0,9.0d0,1.0d0,0.0d0))
  RCONST(519) = (GC_NIT(2.7d-12,350.0d0,12.915d0,9.0d0,1.0d0,0.0d0))
  RCONST(520) = (GC_ALK(2.7d-12,350.0d0,1.412d0,9.0d0,1.0d0,0.0d0))
  RCONST(521) = (GC_NIT(2.7d-12,350.0d0,1.412d0,9.0d0,1.0d0,0.0d0))
  RCONST(522) = (GCARR_ac(2.50d-14,-300.0d0))
  RCONST(523) = (GCARR_ac(1.00d+20,-10000.0d0))
  RCONST(524) = (GCARR_ac(5.88d-12,390.0d0))
  RCONST(525) = (GCARR_ac(1.61d-11,390.0d0))
  RCONST(526) = (GC_EPO_a(4.471d-12,390.0d0,2.28d-20))
  RCONST(527) = (GC_EPO_a(8.77d-12,390.0d0,2.185d-20))
  RCONST(528) = (GC_EPO_a(1.493d-11,390.0d0,2.715d-19))
  RCONST(529) = (GCARR_ac(2.278d-12,200.0d0))
  RCONST(530) = (GCARR_ac(3.40d-12,200.0d0))
  RCONST(531) = (GCARR_ac(7.50d-12,20.0d0))
  RCONST(532) = (GCARR_ac(6.55d+12,-10000.0d0))
  RCONST(533) = (GCARR_ac(8.72d+12,-10000.0d0))
  RCONST(534) = (GCARR_ac(2.64d-13,1300.0d0))
  RCONST(535) = (GCARR_ac(2.64d-13,1300.0d0))
  RCONST(536) = (GC_ALK(2.7d-12,350.0d0,6.092d0,12.0d0,1.0d0,0.0d0))
  RCONST(537) = (GC_NIT(2.7d-12,350.0d0,6.092d0,12.0d0,1.0d0,0.0d0))
  RCONST(538) = (GC_ALK(2.7d-12,350.0d0,4.383d0,12.0d0,1.0d0,0.0d0))
  RCONST(539) = (GC_NIT(2.7d-12,350.0d0,4.383d0,12.0d0,1.0d0,0.0d0))
  RCONST(540) = (GC_EPO_a(2.97d-12,390.0d0,2.715d-19))
  RCONST(541) = (GCARR_ac(9.35d-12,390.0d0))
  RCONST(542) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(543) = (GCARR_ac(2.54d-13,1300.0d0))
  RCONST(544) = (GCARR_ac(1.00d-11,0.0d0))
  RCONST(545) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(546) = (GCARR_ac(2.71d-13,1300.0d0))
  RCONST(547) = (GCARR_ac(2.60d-12,610.0d0))
  RCONST(548) = (GCARR_ac(8.50d-16,-1520.0d0))
  RCONST(549) = (GCARR_ac(4.40d-12,380.0d0))
  RCONST(550) = (GCARR_ac(2.70d-12,470.0d0))
  RCONST(551) = (GCARR_ac(1.40d-15,-2100.0d0))
  RCONST(552) = (GCARR_ac(1.80d-13,-1190.0d0))
  RCONST(553) = (GCARR_ac(1.24d-12,380.0d0))
  RCONST(554) = (5.77d-11)
  RCONST(555) = (GCARR_ac(2.70d-12,470.0d0))
  RCONST(556) = (GCARR_ac(1.39d-11,380.0d0))
  RCONST(557) = (GCARR_ac(2.70d-12,470.0d0))
  RCONST(558) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(559) = (GCARR_ac(1.93d-13,1300.0d0))
  RCONST(560) = (9.00d-12)
  RCONST(561) = (GCARR_ac(2.70d-12,350.0d0))
  RCONST(562) = (GCARR_ac(1.93d-13,1300.0d0))
  RCONST(563) = (9.00d-12)
  RCONST(564) = (GCARR_ac(3.71d-12,983.0d0))
  RCONST(565) = (GCARR_ac(5.00d-12,470.0d0))
  RCONST(566) = (GCARR_ac(8.70d-12,70.0d0))
  RCONST(567) = (GCARR_ac(5.00d-12,470.0d0))
  RCONST(568) = (GCARR_ac(2.00d-12,70.0d0))
  RCONST(569) = (GCARR_ac(2.4d-11,70.0d0))
  RCONST(570) = (GCARR_ac(2.12d-13,1300.0d0))
  RCONST(571) = (GC_ALK(2.7d-12,350.0d0,4.573d0,6.0d0,1.0d0,0.0d0))
  RCONST(572) = (GC_NIT(2.7d-12,350.0d0,4.573d0,6.0d0,1.0d0,0.0d0))
  RCONST(573) = (GCARR_ac(2.12d-13,1300.0d0))
  RCONST(574) = (GCARR_ac(3.14d-12,580.0d0))
  RCONST(575) = (1.66d-11)
  RCONST(576) = (GCARR_ac(2.90d+7,-5297.0d0))
  RCONST(577) = (GC_ALK(2.7d-12,350.0d0,2.985d0,6.0d0,1.0d0,0.0d0))
  RCONST(578) = (GC_NIT(2.7d-12,350.0d0,2.985d0,6.0d0,1.0d0,0.0d0))
  RCONST(579) = (GCARR_ac(8.7d-12,290.0d0))
  RCONST(580) = (GC_PAN_acac(2.591d-28,-6.87d0,1.125d-11,-1.105d0,0.3d0))
  RCONST(581) = (GCARR_ac(3.14d-12,580.0d0))
  RCONST(582) = (GCARR_ac(7.50d-12,290.0d0))
  RCONST(583) = (GC_PAN_acac(2.591d-28,-6.87d0,1.125d-11,-1.105d0,0.3d0))
  RCONST(584) = (4.00d-12)
  RCONST(585) = (GCARR_ac(2.9d-12,500.0d0))
  RCONST(586) = (GCARR_ac(1.58d+16,-13500.0d0))
  RCONST(587) = (2.90d-11)
  RCONST(588) = (4.33d-12)
  RCONST(589) = (1.00d-11)
  RCONST(590) = (2.25d-11)
  RCONST(591) = (3.00d-12)
  RCONST(592) = (3.00d-12)
  RCONST(593) = (1.00d-11)
  RCONST(594) = (3.00d-12)
  RCONST(595) = (GCARR_ac(1.40d-12,-1860.0d0))
  RCONST(596) = (8.00d-13)
  RCONST(597) = (GCARR_ac(1.30d-12,500.0d0))
  RCONST(598) = (GCARR_ac(3.14d-12,580.0d0))
  RCONST(599) = (GCARR_ac(1.55d-12,340.0d0))
  RCONST(600) = (2.91d-11)
  RCONST(601) = (GCARR_ac(1.56d-11,117.0d0))
  RCONST(602) = (GCARR_ac(1.40d-12,700.0d0))
  RCONST(603) = (GCARR_ac(2.60d-12,350.0d0))
  RCONST(604) = (GCARR_abc(9.10d-15,0.0d0,-2580.0d0))
  RCONST(605) = (GCJPLPR_abab(1.10d-28,3.5d+00,8.4d-12,1.75d0,0.5d0))
  RCONST(606) = (GCJPLPR_abab(5.50d-30,0.0d0,8.3d-13,-2.0d0,0.5d0))
  RCONST(607) = (GCARR_abc(1.53d-13,0.0d0,1300.0d0))
  RCONST(608) = (GCARR_abc(2.7d-12,0.0d+00,360.0d0))
  RCONST(609) = (2.3d-12)
  RCONST(610) = (6.00d-13)
  RCONST(611) = (GCARR_abc(9.5d-13,0.0d0,-5988.0d0))
  RCONST(612) = (GCARR_abc(2.5d-14,0.0d0,-300.0d0))
  RCONST(613) = (8.40d-13)
  RCONST(614) = (GCARR_abc(1.90d-12,0.0d+00,190.0d0))
  RCONST(615) = (1.38d-11)
  RCONST(616) = (GCARR_abc(2.3d-12,0.0d0,-193.0d0))
  RCONST(617) = (GCARR_abc(1.8d-12,0.0d0,340.0d0))
  RCONST(618) = (1.7d-11)
  RCONST(619) = (2.91d-13*EXP(1300.0d0/TEMP)*0.82d0)
  RCONST(620) = (GCARR_abc(2.60d-12,0.0d+00,365.0d0))
  RCONST(621) = (2.30d-12)
  RCONST(622) = (GCARR_abc(1.70d-14,0.0d0,220.0d0))
  RCONST(623) = (GCARR_abc(4.20d-14,0.0d0,220.0d0))
  RCONST(624) = (GCARR_abc(4.70d-13,0.0d0,1220.0d0))
  RCONST(625) = (3.8d-12)
  RCONST(626) = (4.7d-11)
  RCONST(627) = (1.4d-11)
  RCONST(628) = (2.0d-11)
  RCONST(629) = (9.2d-18)
  RCONST(630) = (9.9d-11)
  RCONST(631) = (GCARR_abc(5.90d-12,0.0d0,225.0d0))
  RCONST(632) = (2.4d-15)
  RCONST(633) = (GCARR_abc(1.10d-11,0.0d0,340.0d0))
  RCONST(634) = (GCARR_abc(7.50d-12,0.0d0,290.0d0))
  RCONST(635) = (GC_PAN_acac(3.28d-28,-6.87d0,1.125d-11,-1.105d0,0.3d0))
  RCONST(636) = (4.66d-12)
  RCONST(637) = (GC_PAN_abab(1.10d-5,-10100.0d0,1.90d+17,-14100.0d0,0.3d0)*0.67d0)
  RCONST(638) = (1.06d-12)
  RCONST(639) = (7.00d-12)
  RCONST(640) = (GCARR_abc(2.670d-12,0.0d0,365.0d0))
  RCONST(641) = (2.30d-12)
  RCONST(642) = (GCARR_abc(2.24d-13,0.0d0,1300.0d0))
  RCONST(643) = (3.60d-12)
  RCONST(644) = (2.86d-13)
  RCONST(645) = (2.08d-12)
  RCONST(646) = (3.47d-12)
  RCONST(647) = (2.60d-12)
  RCONST(648) = (GCARR_abc(2.670d-13,0.0d0,365.0d0))
  RCONST(649) = (GCARR_abc(2.670d-12,0.0d0,365.0d0))
  RCONST(650) = (5.0d-11)
  RCONST(651) = (8.0d-16)
  RCONST(652) = (1.5d-3)
  RCONST(653) = (5.0d-11)
  RCONST(654) = (8.0d-16)
  RCONST(655) = (1.5d-3)
  RCONST(656) = (GCARR_abc(1.40d-12,0.0d0,700.0d0))
  RCONST(657) = (GCARR_abc(2.60d-12,0.0d0,350.0d0))
  RCONST(658) = (GCARR_abc(1.40d-12,0.0d0,700.0d0))
  RCONST(659) = (GCARR_abc(2.60d-12,0.0d0,350.0d0))
  RCONST(660) = (GCARR_abc(1.40d-12,0.0d0,700.0d0))
  RCONST(661) = (GCARR_abc(2.60d-12,0.0d0,350.0d0))
  RCONST(662) = (HO2uptk1stOrd(State_Het))
  RCONST(663) = (NO2uptk1stOrdAndCloud(State_Het))
  RCONST(664) = (NO3uptk1stOrdAndCloud(State_Het))
  RCONST(665) = (NO3hypsisClonSALA(State_Het))
  RCONST(666) = (NO3hypsisClonSALC(State_Het))
  RCONST(667) = (N2O5uptkByH2O(State_Het))
  RCONST(668) = (N2O5uptkByStratHCl(State_Het))
  RCONST(669) = (N2O5uptkByCloud(State_Het))
  RCONST(670) = (N2O5uptkBySALACl(State_Het))
  RCONST(671) = (N2O5uptkBySALCCl(State_Het))
  RCONST(672) = (OHuptkBySALACl(State_Het))
  RCONST(673) = (OHuptkBySALCCl(State_Het))
  RCONST(674) = (BrNO3uptkByH2O(State_Het))
  RCONST(675) = (BrNO3uptkByHCl(State_Het))
  RCONST(676) = (ClNO3uptkByH2O(State_Het))
  RCONST(677) = (ClNO3uptkByHCl(State_Het))
  RCONST(678) = (ClNO3uptkByHBr(State_Het))
  RCONST(679) = (ClNO3uptkByBrSALA(State_Het))
  RCONST(680) = (ClNO3uptkByBrSALC(State_Het))
  RCONST(681) = (ClNO3uptkBySALACL(State_Het))
  RCONST(682) = (ClNO3uptkBySALCCL(State_Het))
  RCONST(683) = (ClNO2uptkBySALACL(State_Het))
  RCONST(684) = (ClNO2uptkBySALCCL(State_Het))
  RCONST(685) = (ClNO2uptkByHCl(State_Het))
  RCONST(686) = (ClNO2uptkByBrSALA(State_Het))
  RCONST(687) = (ClNO2uptkByBrSALC(State_Het))
  RCONST(688) = (ClNO2uptkByHBr(State_Het))
  RCONST(689) = (HOClUptkByHCl(State_Het))
  RCONST(690) = (HOClUptkByHBr(State_Het))
  RCONST(691) = (HOClUptkBySALACL(State_Het))
  RCONST(692) = (HOClUptkBySALCCL(State_Het))
  RCONST(693) = (HOClUptkByHSO3m(State_Het)+HOClUptkBySO3mm(State_Het))
  RCONST(694) = (HOBrUptkByHBr(State_Het))
  RCONST(695) = (HOBrUptkByHCl(State_Het))
  RCONST(696) = (HOBrUptkBySALACL(State_Het))
  RCONST(697) = (HOBrUptkBySALCCL(State_Het))
  RCONST(698) = (HOBrUptkByBrSALA(State_Het))
  RCONST(699) = (HOBrUptkByBrSALC(State_Het))
  RCONST(700) = (HOBrUptkByHSO3m(State_Het)+HOBrUptkBySO3mm(State_Het))
  RCONST(701) = (O3uptkByHBr(State_Het))
  RCONST(702) = (O3uptkByBrSALA(State_Het))
  RCONST(703) = (O3uptkByBrSALC(State_Het))
  RCONST(704) = (HBrUptkBySALA(State_Het))
  RCONST(705) = (HBrUptkBySALC(State_Het))
  RCONST(706) = (IuptkBySulf1stOrd(SR_MW(ind_HI),0.10_dp,State_Het))
  RCONST(707) = (IuptkBySALA1stOrd(SR_MW(ind_HI),0.10_dp,State_Het))
  RCONST(708) = (IuptkBySALC1stOrd(SR_MW(ind_HI),0.10_dp,State_Het))
  RCONST(709) = (IuptkBySulf1stOrd(SR_MW(ind_I2O2),0.02_dp,State_Het))
  RCONST(710) = (IuptkBySALA1stOrd(SR_MW(ind_I2O2),0.02_dp,State_Het))
  RCONST(711) = (IuptkBySALC1stOrd(SR_MW(ind_I2O2),0.02_dp,State_Het))
  RCONST(712) = (IuptkBySulf1stOrd(SR_MW(ind_I2O3),0.02_dp,State_Het))
  RCONST(713) = (IuptkBySALA1stOrd(SR_MW(ind_I2O3),0.02_dp,State_Het))
  RCONST(714) = (IuptkBySALC1stOrd(SR_MW(ind_I2O3),0.02_dp,State_Het))
  RCONST(715) = (IuptkBySulf1stOrd(SR_MW(ind_I2O4),0.02_dp,State_Het))
  RCONST(716) = (IuptkBySALA1stOrd(SR_MW(ind_I2O4),0.02_dp,State_Het))
  RCONST(717) = (IuptkBySALC1stOrd(SR_MW(ind_I2O4),0.02_dp,State_Het))
  RCONST(718) = (IONO2uptkByH2O(State_Het))
  RCONST(719) = (IbrkdnByAcidBrSALA(SR_MW(ind_IONO),C(ind_IONO),0.02_dp,State_Het))
  RCONST(720) = (IbrkdnByAcidBrSALC(SR_MW(ind_IONO),C(ind_IONO),0.02_dp,State_Het))
  RCONST(721) = (IbrkdnByAcidSALACl(SR_MW(ind_IONO),C(ind_IONO),0.02_dp,State_Het))
  RCONST(722) = (IbrkdnByAcidSALCCl(SR_MW(ind_IONO),C(ind_IONO),0.02_dp,State_Het))
  RCONST(723) = (IbrkdnByAcidBrSALA(SR_MW(ind_IONO2),C(ind_IONO2),0.01_dp,State_Het))
  RCONST(724) = (IbrkdnByAcidBrSALC(SR_MW(ind_IONO2),C(ind_IONO2),0.01_dp,State_Het))
  RCONST(725) = (IbrkdnByAcidSALACl(SR_MW(ind_IONO2),C(ind_IONO2),0.01_dp,State_Het))
  RCONST(726) = (IbrkdnByAcidSALCCl(SR_MW(ind_IONO2),C(ind_IONO2),0.01_dp,State_Het))
  RCONST(727) = (IbrkdnByAcidBrSALA(SR_MW(ind_HOI),C(ind_HOI),0.01_dp,State_Het))
  RCONST(728) = (IbrkdnByAcidBrSALC(SR_MW(ind_HOI),C(ind_HOI),0.01_dp,State_Het))
  RCONST(729) = (IbrkdnByAcidSALACl(SR_MW(ind_HOI),C(ind_HOI),0.01_dp,State_Het))
  RCONST(730) = (IbrkdnByAcidSALCCl(SR_MW(ind_HOI),C(ind_HOI),0.01_dp,State_Het))
  RCONST(731) = (GLYXuptk1stOrd(SR_MW(ind_GLYX),State_Het))
  RCONST(732) = (MGLYuptk1stOrd(SR_MW(ind_MGLY),State_Het))
  RCONST(733) = (IEPOXuptk1stOrd(SR_MW(ind_IEPOXA),.FALSE.,State_Het))
  RCONST(734) = (IEPOXuptk1stOrd(SR_MW(ind_IEPOXB),.FALSE.,State_Het))
  RCONST(735) = (IEPOXuptk1stOrd(SR_MW(ind_IEPOXD),.FALSE.,State_Het))
  RCONST(736) = (VOCuptk1stOrd(SR_MW(ind_LVOC),1.0_dp,State_Het))
  RCONST(737) = (VOCuptk1stOrd(SR_MW(ind_MVKN),5.0E-3_dp,State_Het))
  RCONST(738) = (VOCuptk1stOrd(SR_MW(ind_R4N2),5.0E-3_dp,State_Het))
  RCONST(739) = (VOCuptk1stOrd(SR_MW(ind_MONITS),1.0E-2_dp,State_Het))
  RCONST(740) = (VOCuptk1stOrd(SR_MW(ind_MONITU),1.0E-2_dp,State_Het))
  RCONST(741) = (VOCuptk1stOrd(SR_MW(ind_HONIT),1.0E-2_dp,State_Het))
  RCONST(742) = (MGLYuptk1stOrd(SR_MW(ind_PYAC),State_Het))
  RCONST(743) = (IEPOXuptk1stOrd(SR_MW(ind_HMML),.TRUE.,State_Het))
  RCONST(744) = (VOCuptk1stOrd(SR_MW(ind_IHN1),5.0E-3_dp,State_Het))
  RCONST(745) = (VOCuptk1stOrd(SR_MW(ind_IHN2),5.0E-2_dp,State_Het))
  RCONST(746) = (VOCuptk1stOrd(SR_MW(ind_IHN3),5.0E-3_dp,State_Het))
  RCONST(747) = (VOCuptk1stOrd(SR_MW(ind_IHN4),5.0E-3_dp,State_Het))
  RCONST(748) = (IEPOXuptk1stOrd(SR_MW(ind_ICHE),.FALSE.,State_Het))
  RCONST(749) = (VOCuptk1stOrd(SR_MW(ind_INPD),5.0E-3_dp,State_Het))
  RCONST(750) = (VOCuptk1stOrd(SR_MW(ind_INPB),5.0E-3_dp,State_Het))
  RCONST(751) = (VOCuptk1stOrd(SR_MW(ind_IDN),5.0E-3_dp,State_Het))
  RCONST(752) = (VOCuptk1stOrd(SR_MW(ind_ITCN),5.0E-3_dp,State_Het))
  RCONST(753) = (VOCuptk1stOrd(SR_MW(ind_ITHN),5.0E-3_dp,State_Het))
  RCONST(754) = (VOCuptk1stOrd(SR_MW(ind_MCRHNB),5.0E-3_dp,State_Het))
  RCONST(755) = (VOCuptk1stOrd(SR_MW(ind_MCRHN),5.0E-3_dp,State_Het))
  RCONST(756) = (VOCuptk1stOrd(SR_MW(ind_NPHEN),1.0E-2_dp,State_Het))
  RCONST(757) = (PHOTOL(2))
  RCONST(758) = (PHOTOL(3))
  RCONST(759) = (PHOTOL(1))
  RCONST(760) = (PHOTOL(11))
  RCONST(761) = (PHOTOL(9))
  RCONST(762) = (PHOTOL(10))
  RCONST(763) = (PHOTOL(7))
  RCONST(764) = (PHOTOL(8))
  RCONST(765) = (PHOTOL(16))
  RCONST(766) = (PHOTOL(15))
  RCONST(767) = (PHOTOL(17))
  RCONST(768) = (PHOTOL(18))
  RCONST(769) = (PHOTOL(12))
  RCONST(770) = (PHOTOL(13))
  RCONST(771) = (PHOTOL(14))
  RCONST(772) = (PHOTOL(61))
  RCONST(773) = (PHOTOL(62))
  RCONST(774) = (PHOTOL(59))
  RCONST(775) = (PHOTOL(70))
  RCONST(776) = (PHOTOL(76))
  RCONST(777) = (PHOTOL(77))
  RCONST(778) = (PHOTOL(69))
  RCONST(779) = (PHOTOL(68))
  RCONST(780) = (PHOTOL(72))
  RCONST(781) = (PHOTOL(73))
  RCONST(782) = (PHOTOL(74))
  RCONST(783) = (PHOTOL(71))
  RCONST(784) = (PHOTOL(63))
  RCONST(785) = (PHOTOL(64))
  RCONST(786) = (PHOTOL(65))
  RCONST(787) = (PHOTOL(66))
  RCONST(788) = (PHOTOL(75))
  RCONST(789) = (PHOTOL(79))
  RCONST(790) = (PHOTOL(80))
  RCONST(791) = (PHOTOL(81))
  RCONST(792) = (PHOTOL(82))
  RCONST(793) = (PHOTOL(83))
  RCONST(794) = (PHOTOL(84))
  RCONST(795) = (PHOTOL(85))
  RCONST(796) = (PHOTOL(98))
  RCONST(797) = (PHOTOL(99))
  RCONST(798) = (PHOTOL(23))
  RCONST(799) = (PHOTOL(28))
  RCONST(800) = (PHOTOL(32))
  RCONST(801) = (PHOTOL(29))
  RCONST(802) = (PHOTOL(30))
  RCONST(803) = (PHOTOL(31))
  RCONST(804) = (PHOTOL(56))
  RCONST(805) = (PHOTOL(55))
  RCONST(806) = (PHOTOL(50))
  RCONST(807) = (PHOTOL(43))
  RCONST(808) = (PHOTOL(45))
  RCONST(809) = (PHOTOL(33))
  RCONST(810) = (PHOTOL(22))
  RCONST(811) = (PHOTOL(27))
  RCONST(812) = (PHOTOL(25))
  RCONST(813) = (PHOTOL(26))
  RCONST(814) = (PHOTOL(21))
  RCONST(815) = (PHOTOL(19))
  RCONST(816) = (PHOTOL(20))
  RCONST(817) = (PHOTOL(24))
  RCONST(818) = (PHOTOL(44))
  RCONST(819) = (PHOTOL(42))
  RCONST(820) = (PHOTOL(37))
  RCONST(821) = (PHOTOL(38))
  RCONST(822) = (PHOTOL(39))
  RCONST(823) = (PHOTOL(40))
  RCONST(824) = (PHOTOL(41))
  RCONST(825) = (PHOTOL(47))
  RCONST(826) = (PHOTOL(48))
  RCONST(827) = (PHOTOL(49))
  RCONST(828) = (PHOTOL(46))
  RCONST(829) = (PHOTOL(53))
  RCONST(830) = (PHOTOL(51))
  RCONST(831) = (PHOTOL(54))
  RCONST(832) = (PHOTOL(101))
  RCONST(833) = (PHOTOL(114))
  RCONST(834) = (PHOTOL(115))
  RCONST(835) = (PHOTOL(116))
  RCONST(836) = (PHOTOL(117))
  RCONST(837) = (PHOTOL(118))
  RCONST(838) = (PHOTOL(119))
  RCONST(839) = (PHOTOL(120))
  RCONST(840) = (PHOTOL(121))
  RCONST(841) = (PHOTOL(122))
  RCONST(842) = (PHOTOL(123))
  RCONST(843) = (PHOTOL(124))
  RCONST(844) = (PHOTOL(125))
  RCONST(845) = (PHOTOL(126))
  RCONST(846) = (PHOTOL(127))
  RCONST(847) = (PHOTOL(128))
  RCONST(848) = (PHOTOL(129))
  RCONST(849) = (PHOTOL(103))
  RCONST(850) = (PHOTOL(104))
  RCONST(851) = (PHOTOL(97))
  RCONST(852) = (PHOTOL(36))
  RCONST(853) = (PHOTOL(34))
  RCONST(854) = (PHOTOL(100))
  RCONST(855) = (PHOTOL(6))
  RCONST(856) = (PHOTOL(105))
  RCONST(857) = (PHOTOL(107))
  RCONST(858) = (PHOTOL(111))
  RCONST(859) = (PHOTOL(112))
  RCONST(860) = (PHOTOL(113))
  RCONST(861) = (PHOTOL(130))
  RCONST(862) = (PHOTOL(131))
  RCONST(863) = (PHOTOL(132))
  RCONST(864) = (PHOTOL(133))
  RCONST(865) = (PHOTOL(134))
  RCONST(866) = (PHOTOL(135))
  RCONST(867) = (PHOTOL(136))
  RCONST(868) = (PHOTOL(137))
  RCONST(869) = (PHOTOL(86))
  RCONST(870) = (PHOTOL(87))
  RCONST(871) = (PHOTOL(88))
  RCONST(872) = (PHOTOL(89))
  RCONST(873) = (PHOTOL(90))
  RCONST(874) = (PHOTOL(91))
  RCONST(875) = (PHOTOL(92))
  RCONST(876) = (PHOTOL(93))
  RCONST(877) = (PHOTOL(94))
  RCONST(878) = (PHOTOL(95))
  RCONST(879) = (PHOTOL(96))
  RCONST(880) = (PHOTOL(108))
  RCONST(881) = (PHOTOL(109))
  RCONST(882) = (PHOTOL(110))
  RCONST(883) = (PHOTOL(138))
  RCONST(884) = (PHOTOL(139))
  RCONST(885) = (PHOTOL(140))
  RCONST(886) = (PHOTOL(141))
  RCONST(887) = (PHOTOL(142))
  RCONST(888) = (PHOTOL(143))
  RCONST(889) = (PHOTOL(144))
  RCONST(890) = (PHOTOL(145))
  RCONST(891) = (PHOTOL(146))
  RCONST(892) = (PHOTOL(147))
  RCONST(893) = (PHOTOL(148))
  RCONST(894) = (PHOTOL(149))
  RCONST(895) = (PHOTOL(150))
  RCONST(896) = (PHOTOL(151))
  RCONST(897) = (PHOTOL(152))
  RCONST(898) = (PHOTOL(106))
  RCONST(899) = (PHOTOL(78))
  RCONST(900) = (PHOTOL(153))
  RCONST(901) = (PHOTOL(154))
  RCONST(902) = (PHOTOL(155))
  RCONST(903) = (PHOTOL(156))
  RCONST(904) = (PHOTOL(157))
  RCONST(905) = (PHOTOL(158))
  RCONST(906) = (PHOTOL(159))
  RCONST(907) = (PHOTOL(160))
  RCONST(908) = (PHOTOL(161))
  RCONST(909) = (PHOTOL(162))
  RCONST(910) = (PHOTOL(163))
  RCONST(911) = (PHOTOL(164))
  RCONST(912) = (PHOTOL(165))
  RCONST(913) = (PHOTOL(166))
      
END SUBROUTINE Update_RCONST

! End of Update_RCONST function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Update_PHOTO - function to update photolytical rate constants
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE Update_PHOTO ( )


   USE gckpp_Global

  RCONST(757) = (PHOTOL(2))
  RCONST(758) = (PHOTOL(3))
  RCONST(759) = (PHOTOL(1))
  RCONST(760) = (PHOTOL(11))
  RCONST(761) = (PHOTOL(9))
  RCONST(762) = (PHOTOL(10))
  RCONST(763) = (PHOTOL(7))
  RCONST(764) = (PHOTOL(8))
  RCONST(765) = (PHOTOL(16))
  RCONST(766) = (PHOTOL(15))
  RCONST(767) = (PHOTOL(17))
  RCONST(768) = (PHOTOL(18))
  RCONST(769) = (PHOTOL(12))
  RCONST(770) = (PHOTOL(13))
  RCONST(771) = (PHOTOL(14))
  RCONST(772) = (PHOTOL(61))
  RCONST(773) = (PHOTOL(62))
  RCONST(774) = (PHOTOL(59))
  RCONST(775) = (PHOTOL(70))
  RCONST(776) = (PHOTOL(76))
  RCONST(777) = (PHOTOL(77))
  RCONST(778) = (PHOTOL(69))
  RCONST(779) = (PHOTOL(68))
  RCONST(780) = (PHOTOL(72))
  RCONST(781) = (PHOTOL(73))
  RCONST(782) = (PHOTOL(74))
  RCONST(783) = (PHOTOL(71))
  RCONST(784) = (PHOTOL(63))
  RCONST(785) = (PHOTOL(64))
  RCONST(786) = (PHOTOL(65))
  RCONST(787) = (PHOTOL(66))
  RCONST(788) = (PHOTOL(75))
  RCONST(789) = (PHOTOL(79))
  RCONST(790) = (PHOTOL(80))
  RCONST(791) = (PHOTOL(81))
  RCONST(792) = (PHOTOL(82))
  RCONST(793) = (PHOTOL(83))
  RCONST(794) = (PHOTOL(84))
  RCONST(795) = (PHOTOL(85))
  RCONST(796) = (PHOTOL(98))
  RCONST(797) = (PHOTOL(99))
  RCONST(798) = (PHOTOL(23))
  RCONST(799) = (PHOTOL(28))
  RCONST(800) = (PHOTOL(32))
  RCONST(801) = (PHOTOL(29))
  RCONST(802) = (PHOTOL(30))
  RCONST(803) = (PHOTOL(31))
  RCONST(804) = (PHOTOL(56))
  RCONST(805) = (PHOTOL(55))
  RCONST(806) = (PHOTOL(50))
  RCONST(807) = (PHOTOL(43))
  RCONST(808) = (PHOTOL(45))
  RCONST(809) = (PHOTOL(33))
  RCONST(810) = (PHOTOL(22))
  RCONST(811) = (PHOTOL(27))
  RCONST(812) = (PHOTOL(25))
  RCONST(813) = (PHOTOL(26))
  RCONST(814) = (PHOTOL(21))
  RCONST(815) = (PHOTOL(19))
  RCONST(816) = (PHOTOL(20))
  RCONST(817) = (PHOTOL(24))
  RCONST(818) = (PHOTOL(44))
  RCONST(819) = (PHOTOL(42))
  RCONST(820) = (PHOTOL(37))
  RCONST(821) = (PHOTOL(38))
  RCONST(822) = (PHOTOL(39))
  RCONST(823) = (PHOTOL(40))
  RCONST(824) = (PHOTOL(41))
  RCONST(825) = (PHOTOL(47))
  RCONST(826) = (PHOTOL(48))
  RCONST(827) = (PHOTOL(49))
  RCONST(828) = (PHOTOL(46))
  RCONST(829) = (PHOTOL(53))
  RCONST(830) = (PHOTOL(51))
  RCONST(831) = (PHOTOL(54))
  RCONST(832) = (PHOTOL(101))
  RCONST(833) = (PHOTOL(114))
  RCONST(834) = (PHOTOL(115))
  RCONST(835) = (PHOTOL(116))
  RCONST(836) = (PHOTOL(117))
  RCONST(837) = (PHOTOL(118))
  RCONST(838) = (PHOTOL(119))
  RCONST(839) = (PHOTOL(120))
  RCONST(840) = (PHOTOL(121))
  RCONST(841) = (PHOTOL(122))
  RCONST(842) = (PHOTOL(123))
  RCONST(843) = (PHOTOL(124))
  RCONST(844) = (PHOTOL(125))
  RCONST(845) = (PHOTOL(126))
  RCONST(846) = (PHOTOL(127))
  RCONST(847) = (PHOTOL(128))
  RCONST(848) = (PHOTOL(129))
  RCONST(849) = (PHOTOL(103))
  RCONST(850) = (PHOTOL(104))
  RCONST(851) = (PHOTOL(97))
  RCONST(852) = (PHOTOL(36))
  RCONST(853) = (PHOTOL(34))
  RCONST(854) = (PHOTOL(100))
  RCONST(855) = (PHOTOL(6))
  RCONST(856) = (PHOTOL(105))
  RCONST(857) = (PHOTOL(107))
  RCONST(858) = (PHOTOL(111))
  RCONST(859) = (PHOTOL(112))
  RCONST(860) = (PHOTOL(113))
  RCONST(861) = (PHOTOL(130))
  RCONST(862) = (PHOTOL(131))
  RCONST(863) = (PHOTOL(132))
  RCONST(864) = (PHOTOL(133))
  RCONST(865) = (PHOTOL(134))
  RCONST(866) = (PHOTOL(135))
  RCONST(867) = (PHOTOL(136))
  RCONST(868) = (PHOTOL(137))
  RCONST(869) = (PHOTOL(86))
  RCONST(870) = (PHOTOL(87))
  RCONST(871) = (PHOTOL(88))
  RCONST(872) = (PHOTOL(89))
  RCONST(873) = (PHOTOL(90))
  RCONST(874) = (PHOTOL(91))
  RCONST(875) = (PHOTOL(92))
  RCONST(876) = (PHOTOL(93))
  RCONST(877) = (PHOTOL(94))
  RCONST(878) = (PHOTOL(95))
  RCONST(879) = (PHOTOL(96))
  RCONST(880) = (PHOTOL(108))
  RCONST(881) = (PHOTOL(109))
  RCONST(882) = (PHOTOL(110))
  RCONST(883) = (PHOTOL(138))
  RCONST(884) = (PHOTOL(139))
  RCONST(885) = (PHOTOL(140))
  RCONST(886) = (PHOTOL(141))
  RCONST(887) = (PHOTOL(142))
  RCONST(888) = (PHOTOL(143))
  RCONST(889) = (PHOTOL(144))
  RCONST(890) = (PHOTOL(145))
  RCONST(891) = (PHOTOL(146))
  RCONST(892) = (PHOTOL(147))
  RCONST(893) = (PHOTOL(148))
  RCONST(894) = (PHOTOL(149))
  RCONST(895) = (PHOTOL(150))
  RCONST(896) = (PHOTOL(151))
  RCONST(897) = (PHOTOL(152))
  RCONST(898) = (PHOTOL(106))
  RCONST(899) = (PHOTOL(78))
  RCONST(900) = (PHOTOL(153))
  RCONST(901) = (PHOTOL(154))
  RCONST(902) = (PHOTOL(155))
  RCONST(903) = (PHOTOL(156))
  RCONST(904) = (PHOTOL(157))
  RCONST(905) = (PHOTOL(158))
  RCONST(906) = (PHOTOL(159))
  RCONST(907) = (PHOTOL(160))
  RCONST(908) = (PHOTOL(161))
  RCONST(909) = (PHOTOL(162))
  RCONST(910) = (PHOTOL(163))
  RCONST(911) = (PHOTOL(164))
  RCONST(912) = (PHOTOL(165))
  RCONST(913) = (PHOTOL(166))
      
END SUBROUTINE Update_PHOTO

! End of Update_PHOTO function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



END MODULE gckpp_Rates

