! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Auxiliary Routines File
!
! Generated by KPP-2.5.0 symbolic chemistry Kinetics PreProcessor
!       (https:/github.com/KineticPreProcessor/KPP
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian,   Villanova University, USA
!        R. Sander,   Max-Planck Institute for Chemistry, Mainz, Germany
!        M. Long,     Harvard University, USA
!        H. Lin,      Harvard University, USA
!        R. Yantosca, Harvard University, USA
!
! File                 : gckpp_Util.F90
! Equation file        : gckpp.kpp
! Output root filename : gckpp
!
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE gckpp_Util

  USE gckpp_Parameters
  IMPLICIT NONE

CONTAINS



! User INLINED Utility Functions

! End INLINED Utility Functions

! Utility Functions from KPP_HOME/util/util
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! UTIL - Utility functions
!   Arguments :
!
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

! ****************************************************************
!
! InitSaveData - Opens the data file for writing
!   Parameters :
!
! ****************************************************************

      SUBROUTINE InitSaveData ()

      USE gckpp_Parameters

      open(10, file='gckpp.dat')

      END SUBROUTINE InitSaveData

! End of InitSaveData function
! ****************************************************************

! ****************************************************************
!
! SaveData - Write LOOKAT species in the data file
!   Parameters :
!
! ****************************************************************

      SUBROUTINE SaveData ()

      USE gckpp_Global
      USE gckpp_Monitor

      INTEGER i

      WRITE(10,999) (TIME-TSTART)/3600.D0,  &
      (C(LOOKAT(i))/CFACTOR, i=1,NLOOKAT)
 999  FORMAT(E24.16,100(1X,E24.16))

      END SUBROUTINE SaveData

! End of SaveData function
! ****************************************************************

! ****************************************************************
!
! CloseSaveData - Close the data file
!   Parameters :
!
! ****************************************************************

      SUBROUTINE CloseSaveData ()

      USE gckpp_Parameters

      CLOSE(10)

      END SUBROUTINE CloseSaveData

! End of CloseSaveData function
! ****************************************************************

! ****************************************************************
!
! GenerateMatlab - Generates MATLAB file to load the data file
!   Parameters :
!                It will have a character string to prefix each
!                species name with.
!
! ****************************************************************

      SUBROUTINE GenerateMatlab ( PREFIX )

      USE gckpp_Parameters
      USE gckpp_Global
      USE gckpp_Monitor


      CHARACTER(LEN=8) PREFIX
      INTEGER i

      open(20, file='gckpp.m')
      write(20,*) 'load gckpp.dat;'
      write(20,990) PREFIX
990   FORMAT(A1,'c = gckpp;')
      write(20,*) 'clear gckpp;'
      write(20,991) PREFIX, PREFIX
991   FORMAT(A1,'t=',A1,'c(:,1);')
      write(20,992) PREFIX
992   FORMAT(A1,'c(:,1)=[];')

      do i=1,NLOOKAT
        write(20,993) PREFIX, SPC_NAMES(LOOKAT(i)), PREFIX, i
993     FORMAT(A1,A6,' = ',A1,'c(:,',I2,');')
      end do

      CLOSE(20)

      END SUBROUTINE GenerateMatlab

! End of GenerateMatlab function
! ****************************************************************


! ****************************************************************
!
! Integrator_Update_Options - determine whether to call Update_RCONST,
!    Update_PHOTO, and Update_SUN from within the integrator
!
!   Parameters:
!    option (input)
!        = -1 :  Do not call Update_* functions within the integrator
!        =  0 :  Status quo: Call whichever functions are normally called
!        =  1 :  Call Update_RCONST from within the integrator
!        =  2 :  Call Update_PHOTO from within the integrator
!        =  3 :  Call Update_RCONST and Update_PHOTO from within the int.
!        =  4 :  Call Update_SUN from within the integrator
!        =  5 :  Call Update_SUN and Update_RCONST from within the int.
!        =  6 :  not implemented
!        =  7 :  not implemented
!
!    Do_Update_RCONST (output):
!        =T : Calls Update_RCONST from within the integrator
!        =F : Does not call UPDATE_RCONST from w/in the int.
!
!    Do_Update_PHOTO (output):
!        =T : Calls Update_PHOTO from within the integrator
!        =F : Does not call UPDATE_PHOTO from w/in the int.
!
!    Do_Update_SUN (output):
!        =T : Calls Update_SUN from within the integrator
!        =F : Does not call UPDATE_SUN from w/in the int.
!
! ****************************************************************

      SUBROUTINE Integrator_Update_Options( option,            &
                                            Do_Update_RConst,  &
                                            Do_Update_Photo,   &
                                            Do_Update_Sun     )

      !~~~> Input variables
      INTEGER, INTENT(IN)  :: option

      !~~~> Output variables
      LOGICAL, INTENT(OUT) :: Do_Update_RCONST
      LOGICAL, INTENT(OUT) :: Do_Update_PHOTO
      LOGICAL, INTENT(OUT) :: Do_Update_SUN

      ! Option -1: turn off all Update_* calls within the integrator
      IF ( option == -1 ) THEN
         Do_Update_RCONST = .FALSE.
         Do_Update_PHOTO  = .FALSE.
         Do_Update_SUN    = .FALSE.
         RETURN
      ENDIF

      ! Otherwise determine from the value passed
      Do_Update_RCONST = ( IAND( option, 1 ) > 0 )
      Do_Update_PHOTO  = ( IAND( option, 2 ) > 0 )
      Do_Update_SUN    = ( IAND( option, 4 ) > 0 )

      END SUBROUTINE Integrator_Update_Options

! End of Integrator_Update_Options function
! ****************************************************************
! End Utility Functions from KPP_HOME/util/util
! End of UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Shuffle_user2kpp - function to copy concentrations from USER to KPP
!   Arguments :
!      V_USER    - Concentration of variable species in USER's order
!      V         - Concentrations of variable species (local)
!
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE Shuffle_user2kpp ( V_USER, V )

! V_USER - Concentration of variable species in USER's order
  REAL(kind=dp) :: V_USER(NVAR)
! V - Concentrations of variable species (local)
  REAL(kind=dp) :: V(NVAR)

  V(233) = V_USER(1)
  V(219) = V_USER(2)
  V(215) = V_USER(3)
  V(4) = V_USER(4)
  V(261) = V_USER(5)
  V(113) = V_USER(6)
  V(5) = V_USER(7)
  V(178) = V_USER(8)
  V(155) = V_USER(9)
  V(145) = V_USER(10)
  V(248) = V_USER(11)
  V(193) = V_USER(12)
  V(253) = V_USER(13)
  V(99) = V_USER(14)
  V(60) = V_USER(15)
  V(156) = V_USER(16)
  V(187) = V_USER(17)
  V(105) = V_USER(18)
  V(287) = V_USER(19)
  V(124) = V_USER(20)
  V(132) = V_USER(21)
  V(50) = V_USER(22)
  V(172) = V_USER(23)
  V(274) = V_USER(24)
  V(12) = V_USER(25)
  V(275) = V_USER(26)
  V(276) = V_USER(27)
  V(176) = V_USER(28)
  V(67) = V_USER(29)
  V(95) = V_USER(30)
  V(54) = V_USER(31)
  V(75) = V_USER(32)
  V(157) = V_USER(33)
  V(121) = V_USER(34)
  V(141) = V_USER(35)
  V(142) = V_USER(36)
  V(51) = V_USER(37)
  V(52) = V_USER(38)
  V(53) = V_USER(39)
  V(48) = V_USER(40)
  V(42) = V_USER(41)
  V(46) = V_USER(42)
  V(70) = V_USER(43)
  V(73) = V_USER(44)
  V(1) = V_USER(45)
  V(2) = V_USER(46)
  V(3) = V_USER(47)
  V(262) = V_USER(48)
  V(230) = V_USER(49)
  V(97) = V_USER(50)
  V(39) = V_USER(51)
  V(191) = V_USER(52)
  V(100) = V_USER(53)
  V(43) = V_USER(54)
  V(186) = V_USER(55)
  V(74) = V_USER(56)
  V(68) = V_USER(57)
  V(285) = V_USER(58)
  V(192) = V_USER(59)
  V(72) = V_USER(60)
  V(231) = V_USER(61)
  V(259) = V_USER(62)
  V(277) = V_USER(63)
  V(103) = V_USER(64)
  V(258) = V_USER(65)
  V(6) = V_USER(66)
  V(110) = V_USER(67)
  V(82) = V_USER(68)
  V(129) = V_USER(69)
  V(205) = V_USER(70)
  V(64) = V_USER(71)
  V(114) = V_USER(72)
  V(88) = V_USER(73)
  V(102) = V_USER(74)
  V(175) = V_USER(75)
  V(213) = V_USER(76)
  V(109) = V_USER(77)
  V(232) = V_USER(78)
  V(218) = V_USER(79)
  V(173) = V_USER(80)
  V(55) = V_USER(81)
  V(44) = V_USER(82)
  V(45) = V_USER(83)
  V(273) = V_USER(84)
  V(208) = V_USER(85)
  V(246) = V_USER(86)
  V(272) = V_USER(87)
  V(189) = V_USER(88)
  V(78) = V_USER(89)
  V(79) = V_USER(90)
  V(80) = V_USER(91)
  V(81) = V_USER(92)
  V(271) = V_USER(93)
  V(207) = V_USER(94)
  V(65) = V_USER(95)
  V(83) = V_USER(96)
  V(84) = V_USER(97)
  V(85) = V_USER(98)
  V(188) = V_USER(99)
  V(260) = V_USER(100)
  V(101) = V_USER(101)
  V(281) = V_USER(102)
  V(256) = V_USER(103)
  V(267) = V_USER(104)
  V(264) = V_USER(105)
  V(131) = V_USER(106)
  V(182) = V_USER(107)
  V(120) = V_USER(108)
  V(184) = V_USER(109)
  V(117) = V_USER(110)
  V(125) = V_USER(111)
  V(126) = V_USER(112)
  V(168) = V_USER(113)
  V(265) = V_USER(114)
  V(139) = V_USER(115)
  V(40) = V_USER(116)
  V(47) = V_USER(117)
  V(38) = V_USER(118)
  V(69) = V_USER(119)
  V(170) = V_USER(120)
  V(190) = V_USER(121)
  V(98) = V_USER(122)
  V(236) = V_USER(123)
  V(146) = V_USER(124)
  V(202) = V_USER(125)
  V(106) = V_USER(126)
  V(122) = V_USER(127)
  V(140) = V_USER(128)
  V(229) = V_USER(129)
  V(164) = V_USER(130)
  V(163) = V_USER(131)
  V(159) = V_USER(132)
  V(239) = V_USER(133)
  V(143) = V_USER(134)
  V(127) = V_USER(135)
  V(225) = V_USER(136)
  V(128) = V_USER(137)
  V(224) = V_USER(138)
  V(115) = V_USER(139)
  V(216) = V_USER(140)
  V(136) = V_USER(141)
  V(137) = V_USER(142)
  V(210) = V_USER(143)
  V(242) = V_USER(144)
  V(243) = V_USER(145)
  V(217) = V_USER(146)
  V(185) = V_USER(147)
  V(179) = V_USER(148)
  V(180) = V_USER(149)
  V(183) = V_USER(150)
  V(123) = V_USER(151)
  V(7) = V_USER(152)
  V(56) = V_USER(153)
  V(240) = V_USER(154)
  V(241) = V_USER(155)
  V(177) = V_USER(156)
  V(147) = V_USER(157)
  V(223) = V_USER(158)
  V(96) = V_USER(159)
  V(199) = V_USER(160)
  V(266) = V_USER(161)
  V(86) = V_USER(162)
  V(8) = V_USER(163)
  V(9) = V_USER(164)
  V(220) = V_USER(165)
  V(162) = V_USER(166)
  V(148) = V_USER(167)
  V(209) = V_USER(168)
  V(238) = V_USER(169)
  V(249) = V_USER(170)
  V(10) = V_USER(171)
  V(11) = V_USER(172)
  V(165) = V_USER(173)
  V(221) = V_USER(174)
  V(13) = V_USER(175)
  V(14) = V_USER(176)
  V(15) = V_USER(177)
  V(16) = V_USER(178)
  V(19) = V_USER(179)
  V(20) = V_USER(180)
  V(23) = V_USER(181)
  V(22) = V_USER(182)
  V(24) = V_USER(183)
  V(25) = V_USER(184)
  V(247) = V_USER(185)
  V(203) = V_USER(186)
  V(133) = V_USER(187)
  V(245) = V_USER(188)
  V(66) = V_USER(189)
  V(257) = V_USER(190)
  V(130) = V_USER(191)
  V(160) = V_USER(192)
  V(195) = V_USER(193)
  V(171) = V_USER(194)
  V(227) = V_USER(195)
  V(201) = V_USER(196)
  V(138) = V_USER(197)
  V(222) = V_USER(198)
  V(87) = V_USER(199)
  V(228) = V_USER(200)
  V(263) = V_USER(201)
  V(214) = V_USER(202)
  V(41) = V_USER(203)
  V(174) = V_USER(204)
  V(196) = V_USER(205)
  V(134) = V_USER(206)
  V(149) = V_USER(207)
  V(71) = V_USER(208)
  V(27) = V_USER(209)
  V(150) = V_USER(210)
  V(151) = V_USER(211)
  V(244) = V_USER(212)
  V(116) = V_USER(213)
  V(166) = V_USER(214)
  V(181) = V_USER(215)
  V(226) = V_USER(216)
  V(237) = V_USER(217)
  V(200) = V_USER(218)
  V(152) = V_USER(219)
  V(61) = V_USER(220)
  V(57) = V_USER(221)
  V(169) = V_USER(222)
  V(18) = V_USER(223)
  V(58) = V_USER(224)
  V(59) = V_USER(225)
  V(280) = V_USER(226)
  V(270) = V_USER(227)
  V(283) = V_USER(228)
  V(206) = V_USER(229)
  V(89) = V_USER(230)
  V(17) = V_USER(231)
  V(284) = V_USER(232)
  V(268) = V_USER(233)
  V(269) = V_USER(234)
  V(107) = V_USER(237)
  V(62) = V_USER(238)
  V(279) = V_USER(239)
  V(90) = V_USER(240)
  V(212) = V_USER(241)
  V(211) = V_USER(242)
  V(235) = V_USER(243)
  V(63) = V_USER(244)
  V(118) = V_USER(245)
  V(234) = V_USER(246)
  V(119) = V_USER(247)
  V(204) = V_USER(248)
  V(111) = V_USER(249)
  V(49) = V_USER(250)
  V(197) = V_USER(251)
  V(194) = V_USER(252)
  V(254) = V_USER(253)
  V(112) = V_USER(254)
  V(104) = V_USER(255)
  V(251) = V_USER(256)
  V(198) = V_USER(257)
  V(252) = V_USER(258)
  V(91) = V_USER(259)
  V(92) = V_USER(260)
  V(93) = V_USER(261)
  V(255) = V_USER(262)
  V(250) = V_USER(263)
  V(153) = V_USER(264)
  V(158) = V_USER(265)
  V(167) = V_USER(266)
  V(161) = V_USER(267)
  V(154) = V_USER(268)
  V(94) = V_USER(269)
  V(144) = V_USER(270)
  V(135) = V_USER(271)
  V(278) = V_USER(272)
  V(282) = V_USER(273)
  V(286) = V_USER(278)
  V(108) = V_USER(279)
  V(28) = V_USER(280)
  V(29) = V_USER(281)
  V(30) = V_USER(282)
  V(76) = V_USER(283)
  V(21) = V_USER(284)
  V(77) = V_USER(285)
  V(26) = V_USER(286)

END SUBROUTINE Shuffle_user2kpp

! End of Shuffle_user2kpp function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Shuffle_kpp2user - function to restore concentrations from KPP to USER
!   Arguments :
!      V         - Concentrations of variable species (local)
!      V_USER    - Concentration of variable species in USER's order
!
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE Shuffle_kpp2user ( V, V_USER )

! V - Concentrations of variable species (local)
  REAL(kind=dp) :: V(NVAR)
! V_USER - Concentration of variable species in USER's order
  REAL(kind=dp) :: V_USER(NVAR)

  V_USER(1) = V(233)
  V_USER(2) = V(219)
  V_USER(3) = V(215)
  V_USER(4) = V(4)
  V_USER(5) = V(261)
  V_USER(6) = V(113)
  V_USER(7) = V(5)
  V_USER(8) = V(178)
  V_USER(9) = V(155)
  V_USER(10) = V(145)
  V_USER(11) = V(248)
  V_USER(12) = V(193)
  V_USER(13) = V(253)
  V_USER(14) = V(99)
  V_USER(15) = V(60)
  V_USER(16) = V(156)
  V_USER(17) = V(187)
  V_USER(18) = V(105)
  V_USER(19) = V(287)
  V_USER(20) = V(124)
  V_USER(21) = V(132)
  V_USER(22) = V(50)
  V_USER(23) = V(172)
  V_USER(24) = V(274)
  V_USER(25) = V(12)
  V_USER(26) = V(275)
  V_USER(27) = V(276)
  V_USER(28) = V(176)
  V_USER(29) = V(67)
  V_USER(30) = V(95)
  V_USER(31) = V(54)
  V_USER(32) = V(75)
  V_USER(33) = V(157)
  V_USER(34) = V(121)
  V_USER(35) = V(141)
  V_USER(36) = V(142)
  V_USER(37) = V(51)
  V_USER(38) = V(52)
  V_USER(39) = V(53)
  V_USER(40) = V(48)
  V_USER(41) = V(42)
  V_USER(42) = V(46)
  V_USER(43) = V(70)
  V_USER(44) = V(73)
  V_USER(45) = V(1)
  V_USER(46) = V(2)
  V_USER(47) = V(3)
  V_USER(48) = V(262)
  V_USER(49) = V(230)
  V_USER(50) = V(97)
  V_USER(51) = V(39)
  V_USER(52) = V(191)
  V_USER(53) = V(100)
  V_USER(54) = V(43)
  V_USER(55) = V(186)
  V_USER(56) = V(74)
  V_USER(57) = V(68)
  V_USER(58) = V(285)
  V_USER(59) = V(192)
  V_USER(60) = V(72)
  V_USER(61) = V(231)
  V_USER(62) = V(259)
  V_USER(63) = V(277)
  V_USER(64) = V(103)
  V_USER(65) = V(258)
  V_USER(66) = V(6)
  V_USER(67) = V(110)
  V_USER(68) = V(82)
  V_USER(69) = V(129)
  V_USER(70) = V(205)
  V_USER(71) = V(64)
  V_USER(72) = V(114)
  V_USER(73) = V(88)
  V_USER(74) = V(102)
  V_USER(75) = V(175)
  V_USER(76) = V(213)
  V_USER(77) = V(109)
  V_USER(78) = V(232)
  V_USER(79) = V(218)
  V_USER(80) = V(173)
  V_USER(81) = V(55)
  V_USER(82) = V(44)
  V_USER(83) = V(45)
  V_USER(84) = V(273)
  V_USER(85) = V(208)
  V_USER(86) = V(246)
  V_USER(87) = V(272)
  V_USER(88) = V(189)
  V_USER(89) = V(78)
  V_USER(90) = V(79)
  V_USER(91) = V(80)
  V_USER(92) = V(81)
  V_USER(93) = V(271)
  V_USER(94) = V(207)
  V_USER(95) = V(65)
  V_USER(96) = V(83)
  V_USER(97) = V(84)
  V_USER(98) = V(85)
  V_USER(99) = V(188)
  V_USER(100) = V(260)
  V_USER(101) = V(101)
  V_USER(102) = V(281)
  V_USER(103) = V(256)
  V_USER(104) = V(267)
  V_USER(105) = V(264)
  V_USER(106) = V(131)
  V_USER(107) = V(182)
  V_USER(108) = V(120)
  V_USER(109) = V(184)
  V_USER(110) = V(117)
  V_USER(111) = V(125)
  V_USER(112) = V(126)
  V_USER(113) = V(168)
  V_USER(114) = V(265)
  V_USER(115) = V(139)
  V_USER(116) = V(40)
  V_USER(117) = V(47)
  V_USER(118) = V(38)
  V_USER(119) = V(69)
  V_USER(120) = V(170)
  V_USER(121) = V(190)
  V_USER(122) = V(98)
  V_USER(123) = V(236)
  V_USER(124) = V(146)
  V_USER(125) = V(202)
  V_USER(126) = V(106)
  V_USER(127) = V(122)
  V_USER(128) = V(140)
  V_USER(129) = V(229)
  V_USER(130) = V(164)
  V_USER(131) = V(163)
  V_USER(132) = V(159)
  V_USER(133) = V(239)
  V_USER(134) = V(143)
  V_USER(135) = V(127)
  V_USER(136) = V(225)
  V_USER(137) = V(128)
  V_USER(138) = V(224)
  V_USER(139) = V(115)
  V_USER(140) = V(216)
  V_USER(141) = V(136)
  V_USER(142) = V(137)
  V_USER(143) = V(210)
  V_USER(144) = V(242)
  V_USER(145) = V(243)
  V_USER(146) = V(217)
  V_USER(147) = V(185)
  V_USER(148) = V(179)
  V_USER(149) = V(180)
  V_USER(150) = V(183)
  V_USER(151) = V(123)
  V_USER(152) = V(7)
  V_USER(153) = V(56)
  V_USER(154) = V(240)
  V_USER(155) = V(241)
  V_USER(156) = V(177)
  V_USER(157) = V(147)
  V_USER(158) = V(223)
  V_USER(159) = V(96)
  V_USER(160) = V(199)
  V_USER(161) = V(266)
  V_USER(162) = V(86)
  V_USER(163) = V(8)
  V_USER(164) = V(9)
  V_USER(165) = V(220)
  V_USER(166) = V(162)
  V_USER(167) = V(148)
  V_USER(168) = V(209)
  V_USER(169) = V(238)
  V_USER(170) = V(249)
  V_USER(171) = V(10)
  V_USER(172) = V(11)
  V_USER(173) = V(165)
  V_USER(174) = V(221)
  V_USER(175) = V(13)
  V_USER(176) = V(14)
  V_USER(177) = V(15)
  V_USER(178) = V(16)
  V_USER(179) = V(19)
  V_USER(180) = V(20)
  V_USER(181) = V(23)
  V_USER(182) = V(22)
  V_USER(183) = V(24)
  V_USER(184) = V(25)
  V_USER(185) = V(247)
  V_USER(186) = V(203)
  V_USER(187) = V(133)
  V_USER(188) = V(245)
  V_USER(189) = V(66)
  V_USER(190) = V(257)
  V_USER(191) = V(130)
  V_USER(192) = V(160)
  V_USER(193) = V(195)
  V_USER(194) = V(171)
  V_USER(195) = V(227)
  V_USER(196) = V(201)
  V_USER(197) = V(138)
  V_USER(198) = V(222)
  V_USER(199) = V(87)
  V_USER(200) = V(228)
  V_USER(201) = V(263)
  V_USER(202) = V(214)
  V_USER(203) = V(41)
  V_USER(204) = V(174)
  V_USER(205) = V(196)
  V_USER(206) = V(134)
  V_USER(207) = V(149)
  V_USER(208) = V(71)
  V_USER(209) = V(27)
  V_USER(210) = V(150)
  V_USER(211) = V(151)
  V_USER(212) = V(244)
  V_USER(213) = V(116)
  V_USER(214) = V(166)
  V_USER(215) = V(181)
  V_USER(216) = V(226)
  V_USER(217) = V(237)
  V_USER(218) = V(200)
  V_USER(219) = V(152)
  V_USER(220) = V(61)
  V_USER(221) = V(57)
  V_USER(222) = V(169)
  V_USER(223) = V(18)
  V_USER(224) = V(58)
  V_USER(225) = V(59)
  V_USER(226) = V(280)
  V_USER(227) = V(270)
  V_USER(228) = V(283)
  V_USER(229) = V(206)
  V_USER(230) = V(89)
  V_USER(231) = V(17)
  V_USER(232) = V(284)
  V_USER(233) = V(268)
  V_USER(234) = V(269)
  V_USER(237) = V(107)
  V_USER(238) = V(62)
  V_USER(239) = V(279)
  V_USER(240) = V(90)
  V_USER(241) = V(212)
  V_USER(242) = V(211)
  V_USER(243) = V(235)
  V_USER(244) = V(63)
  V_USER(245) = V(118)
  V_USER(246) = V(234)
  V_USER(247) = V(119)
  V_USER(248) = V(204)
  V_USER(249) = V(111)
  V_USER(250) = V(49)
  V_USER(251) = V(197)
  V_USER(252) = V(194)
  V_USER(253) = V(254)
  V_USER(254) = V(112)
  V_USER(255) = V(104)
  V_USER(256) = V(251)
  V_USER(257) = V(198)
  V_USER(258) = V(252)
  V_USER(259) = V(91)
  V_USER(260) = V(92)
  V_USER(261) = V(93)
  V_USER(262) = V(255)
  V_USER(263) = V(250)
  V_USER(264) = V(153)
  V_USER(265) = V(158)
  V_USER(266) = V(167)
  V_USER(267) = V(161)
  V_USER(268) = V(154)
  V_USER(269) = V(94)
  V_USER(270) = V(144)
  V_USER(271) = V(135)
  V_USER(272) = V(278)
  V_USER(273) = V(282)
  V_USER(278) = V(286)
  V_USER(279) = V(108)
  V_USER(280) = V(28)
  V_USER(281) = V(29)
  V_USER(282) = V(30)
  V_USER(283) = V(76)
  V_USER(284) = V(21)
  V_USER(285) = V(77)
  V_USER(286) = V(26)

END SUBROUTINE Shuffle_kpp2user

! End of Shuffle_kpp2user function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! GetMass - compute total mass of selected atoms
!   Arguments :
!      CL        - Concentration of all species (local)
!      Mass      - value of mass balance
!
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE GetMass ( CL, Mass )

! CL - Concentration of all species (local)
  REAL(kind=dp) :: CL(NSPEC)
! Mass - value of mass balance
  REAL(kind=dp) :: Mass(1)


END SUBROUTINE GetMass

! End of GetMass function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! ComputeFamilies - function to calculate user-defined Prod/Loss families
!   Arguments :
!      V         - Concentrations of variable species (local)
!      FAM       - Accumulated user-defined prod/loss families.
!
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE ComputeFamilies ( V, FAM )

! V - Concentrations of variable species (local)
  REAL(kind=dp) :: V(NVAR)
! FAM - Accumulated user-defined prod/loss families.
  REAL(kind=dp) :: FAM(NFAM)


! Computation of prod/loss families
  FAM(1) = V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)&
             &+0.15*V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+2*V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)&
             &+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(33)+V(31)+V(31)&
             &+V(31)+V(33)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(33)+V(31)+V(33)+V(31)+V(31)+V(33)+V(31)+V(31)+V(31)+V(31)&
             &+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(33)+V(31)&
             &+0.69*V(31)+V(31)+V(31)+V(31)+0.438*V(31)+0.13*V(33)+V(33)+V(31)+V(33)+0.13*V(31)+V(31)+V(33)+0.02*V(31)+0.13&
             &*V(31)+0.995*V(31)+V(31)+V(31)+0.15*V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+2*V(31)+V(31)+0.3*V(31)+V(31)&
             &+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+V(31)+0.00999999*V(31)
  FAM(2) = V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+2*V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)&
             &+V(32)+V(32)+V(32)+2*V(32)+V(32)+2*V(32)+2*V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(36)+V(36)+V(36)+2&
             &*V(32)+V(32)+V(32)+2*V(32)+V(32)+2*V(32)+2*V(32)+V(32)+V(32)+2*V(32)+V(32)+0.67*V(32)+0.65*V(32)+0.56*V(32)&
             &+0.17*V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+2*V(32)+2*V(32)+2*V(32)+2*V(32)+2*V(32)+2*V(32)+2*V(32)+2*V(32)&
             &+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+2*V(32)+2*V(32)+V(32)+V(32)&
             &+0.00100005*V(32)+5.96046e-08*V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+1.258*V(32)&
             &+1.5*V(32)+V(32)+1.5*V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+0.5*V(32)+V(32)+2*V(32)+2*V(32)+V(32)&
             &+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)&
             &+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+2*V(32)+2*V(32)+2*V(32)+3*V(32)+3*V(32)+3&
             &*V(32)+4*V(32)+4*V(32)+4*V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)&
             &+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+V(32)+2*V(32)+V(32)+V(32)+V(32)+V(32)+2*V(32)+V(32)+2&
             &*V(32)+V(32)+V(32)+2*V(32)
  FAM(3) = V(33)+V(33)+0.05*V(33)+0.56*V(33)+0.505*V(33)+V(33)+2*V(33)+V(33)+2*V(33)+V(33)+0.5*V(33)+V(33)+V(33)+V(33)&
             &+V(33)+V(33)+V(33)+V(33)+0.14*V(37)+0.14*V(37)+0.14*V(37)+0.407*V(37)+0.6*V(33)+0.9*V(33)+0.6*V(33)+0.9*V(33)&
             &+V(33)+V(33)+V(33)+0.5*V(33)+0.5*V(33)+0.55*V(33)+0.55*V(33)+0.45*V(33)+0.37*V(33)+0.42*V(33)+0.23*V(33)+0.638&
             &*V(33)+1.5*V(33)+V(33)+0.25*V(33)+0.33*V(33)+0.036*V(33)+0.054*V(33)+V(33)+0.13*V(33)+0.2*V(33)+V(33)+0.2&
             &*V(33)+0.13*V(33)+0.52*V(33)+0.8*V(33)+2*V(33)+0.6*V(33)+0.6*V(33)+0.539*V(33)+0.33*V(33)+0.33*V(33)+0.18&
             &*V(37)+0.036*V(33)+0.12*V(33)+0.68*V(33)+0.77*V(33)+0.25*V(33)+0.75*V(33)+V(33)+2*V(33)+0.84*V(33)+0.507*V(33)&
             &+0.325*V(33)+0.488*V(33)+V(33)+0.86*V(33)+0.65*V(33)+0.25*V(33)+V(33)+0.888*V(33)+V(33)+V(33)+0.364*V(33)+0.2&
             &*V(33)+0.27*V(33)+0.3*V(33)+0.25*V(33)+0.5*V(33)+0.25*V(33)+0.5*V(33)+V(33)+V(33)+0.88*V(33)+V(33)+V(33)+V(33)&
             &+V(33)+2*V(33)+2*V(33)+V(33)+V(33)+V(33)+V(33)+V(33)+V(33)+V(33)+V(33)+V(33)+0.5*V(33)+0.571*V(33)+0.875*V(33)&
             &+0.77*V(33)+V(33)+V(33)+0.888*V(33)+0.818*V(33)+V(33)+V(33)+0.55*V(33)+0.55*V(33)+0.839*V(33)+V(33)+0.122&
             &*V(33)+V(33)+V(33)+V(33)+V(33)
  FAM(4) = V(34)+V(34)+V(34)
  FAM(5) = V(35)+V(35)+V(35)+V(35)+2*V(35)+V(35)+V(35)+V(35)
  FAM(6) = V(36)+V(36)+V(36)+V(36)+V(36)
  FAM(7) = V(37)+V(37)+0.06*V(37)+0.06*V(37)+V(37)+0.02*V(37)+0.02*V(37)+0.02*V(37)+0.013*V(37)+0.045*V(37)

END SUBROUTINE ComputeFamilies

! End of ComputeFamilies function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

! Get_OHreactivity - returns the OH reactivity
! The OH reactivity is defined as the inverse of its lifetime.
! This routine was auto-generated using script OHreact_parser.py.
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE Get_OHreactivity ( CC, RR, OHreact )

! CC - Concentrations of species (local)
  REAL(kind=dp) :: CC(NSPEC)
! RR - reaction rates (local)
  REAL(kind=dp) :: RR(NREACT)
! OHreact - OH reactivity [s-1]
  REAL(kind=dp) :: OHreact

  OHreact = RR(12)*CC(85) + RR(14)*CC(269) + 2*RR(18) + 2*RR(19) + RR(20)*CC(281) &
          + RR(21)*CC(208) + RR(24)*CC(258) + RR(25)*CC(186) + RR(31)*CC(263) + RR(32)*CC(134) &
          + RR(33)*CC(134) + RR(34)*CC(193) + RR(35)*CC(193) + RR(36)*CC(262) + RR(37)*CC(270) &
          + RR(38)*CC(260) + RR(39)*CC(280) + RR(40)*CC(188) + RR(43)*CC(101) + RR(46)*CC(283) &
          + RR(49)*CC(207) + RR(50)*CC(214) + RR(53)*CC(261) + RR(58)*CC(157) + RR(62)*CC(121) &
          + RR(63)*CC(121) + RR(67)*CC(113) + RR(77)*CC(198) + RR(78)*CC(215) + RR(79)*CC(255) &
          + RR(84)*CC(219) + RR(93)*CC(222) + RR(103)*CC(129) + RR(104)*CC(154) + RR(114)*CC(254) &
          + RR(116)*CC(232) + RR(117)*CC(232) + RR(119)*CC(218) + RR(120)*CC(228) + RR(123)*CC(246) &
          + RR(124)*CC(246) + RR(131)*CC(112) + RR(132)*CC(109) + RR(133)*CC(92) + RR(134)*CC(93) &
          + RR(135)*CC(91) + RR(136)*CC(94) + RR(137)*CC(111) + RR(138)*CC(23) + RR(139)*CC(66) &
          + RR(164)*CC(82) + RR(165)*CC(82) + RR(167)*CC(286) + RR(171)*CC(272) + RR(176)*CC(124) &
          + RR(179)*CC(274) + RR(188)*CC(74) + RR(189)*CC(70) + RR(190)*CC(97) + RR(202)*CC(288) &
          + RR(203)*CC(284) + RR(208)*CC(62) + RR(243)*CC(192) + RR(245)*CC(277) + RR(246)*CC(277) &
          + RR(247)*CC(107) + RR(248)*CC(72) + RR(249)*CC(271) + RR(250)*CC(267) + RR(251)*CC(231) &
          + RR(252)*CC(259) + RR(253)*CC(100) + RR(254)*CC(73) + RR(255)*CC(68) + RR(256)*CC(39) &
          + RR(257)*CC(81) + RR(258)*CC(79) + RR(259)*CC(80) + RR(260)*CC(78) + RR(326)*CC(139) &
          + RR(327)*CC(65) + RR(328)*CC(264) + RR(335)*CC(43) + RR(336)*CC(205) + RR(337)*CC(194) &
          + RR(351)*CC(150) + RR(352)*CC(151) + RR(362)*CC(165) + RR(370)*CC(119) + RR(384)*CC(174) &
          + RR(385)*CC(196) + RR(391)*CC(131) + RR(392)*CC(87) + RR(393)*CC(88) + RR(394)*CC(86) &
          + RR(395)*CC(89) + RR(397)*CC(220) + RR(398)*CC(220) + RR(399)*CC(220) + RR(400)*CC(220) &
          + RR(427)*CC(182) + RR(428)*CC(184) + RR(429)*CC(125) + RR(430)*CC(126) + RR(431)*CC(189) &
          + RR(432)*CC(170) + RR(433)*CC(106) + RR(434)*CC(153) + RR(435)*CC(153) + RR(436)*CC(158) &
          + RR(437)*CC(158) + RR(438)*CC(153) + RR(439)*CC(158) + RR(440)*CC(167) + RR(441)*CC(161) &
          + RR(454)*CC(115) + RR(455)*CC(127) + RR(456)*CC(127) + RR(457)*CC(128) + RR(458)*CC(128) &
          + RR(477)*CC(136) + RR(478)*CC(136) + RR(479)*CC(137) + RR(480)*CC(137) + RR(481)*CC(216) &
          + RR(482)*CC(216) + RR(483)*CC(210) + RR(484)*CC(210) + RR(485)*CC(216) + RR(486)*CC(210) &
          + RR(524)*CC(177) + RR(525)*CC(147) + RR(526)*CC(177) + RR(527)*CC(147) + RR(528)*CC(147) &
          + RR(529)*CC(177) + RR(530)*CC(147) + RR(531)*CC(147) + RR(540)*CC(236) + RR(541)*CC(236) &
          + RR(544)*CC(239) + RR(547)*CC(244) + RR(549)*CC(247) + RR(550)*CC(247) + RR(553)*CC(237) &
          + RR(554)*CC(226) + RR(555)*CC(227) + RR(556)*CC(195) + RR(557)*CC(171) + RR(564)*CC(160) &
          + RR(565)*CC(152) + RR(566)*CC(116) + RR(567)*CC(181) + RR(568)*CC(166) + RR(569)*CC(130) &
          + RR(575)*CC(133) + RR(587)*CC(149) + RR(588)*CC(84) + RR(589)*CC(202) + RR(590)*CC(122) &
          + RR(591)*CC(140) + RR(592)*CC(159) + RR(593)*CC(209) + RR(594)*CC(238) + RR(596)*CC(104) &
          + RR(597)*CC(83) + RR(599)*CC(168) + RR(600)*CC(168) + RR(601)*CC(18) + RR(605)*CC(75) &
          + RR(606)*CC(54) + RR(613)*CC(64) + RR(614)*CC(114) + RR(615)*CC(114) + RR(616)*CC(60) &
          + RR(617)*CC(76) + RR(618)*CC(77) + RR(624)*CC(118) + RR(626)*CC(110) + RR(628)*CC(138) &
          + RR(631)*CC(99) + RR(636)*CC(67) + RR(638)*CC(95) + RR(643)*CC(105) + RR(646)*CC(206) &
          + RR(650)*CC(155) + RR(653)*CC(145) + RR(672)*CC(278) + RR(673)*CC(282)

END SUBROUTINE Get_OHreactivity
! End of Get_OHreactivity subroutine
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

END MODULE gckpp_Util
